import _regeneratorRuntime from 'babel-runtime/regenerator';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _get from 'lodash/get';
import _isEmpty from 'lodash/isEmpty';

var _this = this;

export var isDisablePreChatForm = function isDisablePreChatForm(setting) {
  var value = _get(setting, 'chatboxThemeSettings.preChatForm.enable');
  if (typeof value === 'undefined' || value === false) return true;
  return false;
};

export var isTriggerWelcomeBackMessageWhenClickIcon = function isTriggerWelcomeBackMessageWhenClickIcon(setting) {
  return _get(setting, 'screenWelcomeBackMessages.trigger.triggerType') === 'click-icon';
};

export var isTriggerWelcomeMessageWhenClickIcon = function isTriggerWelcomeMessageWhenClickIcon(setting) {
  var value = _get(setting, 'screenWelcomeMessages.trigger.triggerType');
  return typeof value === 'undefined' || value === 'click-icon';
};

export var isTriggerWelcomeBackMessageWhenVisitPage = function isTriggerWelcomeBackMessageWhenVisitPage(setting) {
  return _get(setting, 'screenWelcomeBackMessages.trigger.triggerType') === 'visit-page';
};

export var isTriggerWelcomeMessageWhenVisitPage = function isTriggerWelcomeMessageWhenVisitPage(setting) {
  return _get(setting, 'screenWelcomeMessages.trigger.triggerType') === 'visit-page';
};

export var isLastTimeMoreThanOneDay = function isLastTimeMoreThanOneDay(lastTime) {
  if (_isEmpty(lastTime)) return false;

  var fromDate = typeof lastTime === 'string' ? new Date(lastTime) : lastTime;
  return Math.abs(fromDate - new Date()) >= 1000 * 60 * 60 * 24;
};

export var isLastTimeMoreThanAPeriod = function isLastTimeMoreThanAPeriod(lastTime, duration) {
  if (typeof lastTime === 'undefined' || typeof duration === 'undefined') return false;
  var fromDate = typeof lastTime === 'string' ? new Date(lastTime) : lastTime;
  return Math.abs(fromDate - new Date()) >= duration;
};

var DURATION_TO_RESET_DECISION = 30 * 60 * 1000;
// const DURATION_TO_RESET_DECISION = 20 * 1000;

export var getCurrentScreen = function getCurrentScreen(props) {
  var isInConfigurationMode = props.isInConfigurationMode,
      isStartConversationFromAdmin = props.isStartConversationFromAdmin,
      demoData = props.demoData,
      setting = props.setting,
      alreadySubmitUserInfo = props.alreadySubmitUserInfo,
      isInOperatingHours = props.isInOperatingHours,
      isEnableAutomation = props.isEnableAutomation,
      lastTimeAcceptChattingWithBot = props.lastTimeAcceptChattingWithBot,
      lastTimeAcceptLeavingOfflineMessage = props.lastTimeAcceptLeavingOfflineMessage,
      lastTimeReceivingMessageFromAdmin = props.lastTimeReceivingMessageFromAdmin;


  if (isInConfigurationMode) {
    return demoData.currentScreen === 'welcome_messages' ? 'conversation' : demoData.currentScreen;
  }

  if (isStartConversationFromAdmin && lastTimeReceivingMessageFromAdmin && !isLastTimeMoreThanAPeriod(lastTimeReceivingMessageFromAdmin, DURATION_TO_RESET_DECISION)) {
    return 'conversation';
  }

  if (!isDisablePreChatForm(setting) && !alreadySubmitUserInfo) {
    return 'form';
  }

  if (!isInOperatingHours) {
    if (isEnableAutomation && (!lastTimeAcceptChattingWithBot || isLastTimeMoreThanAPeriod(lastTimeAcceptChattingWithBot, DURATION_TO_RESET_DECISION))) {
      return 'off_with_chatbot';
    }

    if (!isEnableAutomation && (!lastTimeAcceptLeavingOfflineMessage || isLastTimeMoreThanAPeriod(lastTimeAcceptLeavingOfflineMessage, DURATION_TO_RESET_DECISION))) {
      return 'off';
    }
  }
  return 'conversation';
};

export var wait = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(duration) {
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            return _context.abrupt('return', new Promise(function (resolve, reject) {
              setTimeout(resolve, duration);
            }));

          case 1:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, _this);
  }));

  return function wait(_x) {
    return _ref.apply(this, arguments);
  };
}();

export var retryFuncWithFibonancies = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(funcChecking, fiboNumbers) {
    var fibonanciNumbers, currentWaitTimeIndex;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            fibonanciNumbers = fiboNumbers || [0.5, 1, 2, 3, 5, 8, 13];
            currentWaitTimeIndex = 0;

          case 2:
            _context2.next = 4;
            return wait(fibonanciNumbers[currentWaitTimeIndex] * 1000);

          case 4:
            ++currentWaitTimeIndex;

          case 5:
            if (!funcChecking() && currentWaitTimeIndex < fibonanciNumbers.length - 1) {
              _context2.next = 2;
              break;
            }

          case 6:
            if (funcChecking()) {
              _context2.next = 8;
              break;
            }

            throw new Error();

          case 8:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, _this);
  }));

  return function retryFuncWithFibonancies(_x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();