import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
// Custom from lib react-detect-offline
import { Component, isValidElement, Children, createElement, PureComponent } from 'react';
import PropTypes from 'prop-types';

var inBrowser = typeof navigator !== 'undefined';

// these browsers don't fully support navigator.onLine, so we need to use a polling backup
var unsupportedUserAgentsPattern = /Windows.*Chrome|Windows.*Firefox|Linux.*Chrome/;

var ping = function ping(_ref) {
  var url = _ref.url,
      timeout = _ref.timeout;

  return new Promise(function (resolve) {
    var isOnline = function isOnline() {
      return resolve(true);
    };
    var isOffline = function isOffline() {
      return resolve(false);
    };

    var xhr = new XMLHttpRequest();

    xhr.onerror = isOffline;
    xhr.ontimeout = isOffline;
    xhr.onreadystatechange = function () {
      if (xhr.readyState === xhr.HEADERS_RECEIVED) {
        if (xhr.status) {
          isOnline();
        } else {
          isOffline();
        }
      }
    };

    xhr.open('HEAD', url);
    xhr.timeout = timeout;
    xhr.send();
  });
};

var propTypes = {
  children: PropTypes.node,
  onChange: PropTypes.func,
  polling: PropTypes.oneOfType([PropTypes.shape({
    url: PropTypes.string,
    interval: PropTypes.number,
    timeout: PropTypes.number
  }), PropTypes.bool]),
  wrapperType: PropTypes.string
};

var defaultProps = {
  polling: true,
  wrapperType: 'span'
};

var defaultPollingConfig = {
  enabled: inBrowser && unsupportedUserAgentsPattern.test(navigator.userAgent),
  url: 'https://api.boostcrm.io/',
  timeout: 5000,
  interval: 5000
};

// base class that detects offline/online changes

var Base = function (_PureComponent) {
  _inherits(Base, _PureComponent);

  function Base() {
    _classCallCheck(this, Base);

    var _this = _possibleConstructorReturn(this, _PureComponent.call(this));

    _this.state = {
      online: inBrowser && typeof navigator.onLine === 'boolean' ? navigator.onLine : true
    };
    // bind event handlers
    _this.goOnline = _this.goOnline.bind(_this);
    _this.goOffline = _this.goOffline.bind(_this);
    return _this;
  }

  Base.prototype.componentDidMount = function componentDidMount() {
    window.addEventListener('online', this.goOnline);
    window.addEventListener('offline', this.goOffline);

    if (this.getPollingConfig().enabled) {
      this.startPolling();
    }
  };

  Base.prototype.componentWillUnmount = function componentWillUnmount() {
    window.removeEventListener('online', this.goOnline);
    window.removeEventListener('offline', this.goOffline);

    if (this.pollingId) {
      this.stopPolling();
    }
  };

  Base.prototype.renderChildren = function renderChildren() {
    var _props = this.props,
        children = _props.children,
        wrapperType = _props.wrapperType;

    // usual case: one child that is a react Element

    if (isValidElement(children)) {
      return children;
    }

    // no children
    if (!children) {
      return null;
    }

    // string children, multiple children, or something else
    return createElement.apply(undefined, [wrapperType, {}].concat(Children.toArray(children)));
  };

  Base.prototype.getPollingConfig = function getPollingConfig() {
    switch (this.props.polling) {
      case true:
        return defaultPollingConfig;
      case false:
        return { enabled: false };
      default:
        return Object.assign({}, defaultPollingConfig, this.props.polling);
    }
  };

  Base.prototype.goOnline = function goOnline() {
    if (!this.state.online) {
      this.callOnChangeHandler(true);
      this.setState({ online: true });
    }
  };

  Base.prototype.goOffline = function goOffline() {
    if (this.state.online) {
      this.callOnChangeHandler(false);
      this.setState({ online: false });
    }
  };

  Base.prototype.callOnChangeHandler = function callOnChangeHandler(online) {
    if (this.props.onChange) {
      this.props.onChange(online);
    }
  };

  Base.prototype.startPolling = function startPolling() {
    var _this2 = this;

    var _getPollingConfig = this.getPollingConfig(),
        interval = _getPollingConfig.interval;

    this.pollingId = setInterval(function () {
      var _getPollingConfig2 = _this2.getPollingConfig(),
          url = _getPollingConfig2.url,
          timeout = _getPollingConfig2.timeout;

      ping({ url: url, timeout: timeout }).then(function (online) {
        online ? _this2.goOnline() : _this2.goOffline();
      });
    }, interval);
  };

  Base.prototype.stopPolling = function stopPolling() {
    clearInterval(this.pollingId);
  };

  Base.prototype.render = function render() {
    return null;
  };

  return Base;
}(PureComponent);

Base.propTypes = process.env.NODE_ENV !== "production" ? propTypes : {};
Base.defaultProps = defaultProps;

export var NetworkDetector = function (_Base) {
  _inherits(NetworkDetector, _Base);

  function NetworkDetector() {
    _classCallCheck(this, NetworkDetector);

    return _possibleConstructorReturn(this, _Base.apply(this, arguments));
  }

  NetworkDetector.prototype.render = function render() {
    return null;
  };

  return NetworkDetector;
}(Base);
NetworkDetector.propTypes = process.env.NODE_ENV !== "production" ? propTypes : {};
NetworkDetector.defaultProps = defaultProps;