import _extends from 'babel-runtime/helpers/extends';
import _find from 'lodash/find';
import _get from 'lodash/get';
import React, { useContext, useEffect, Component, Suspense, lazy, useRef } from 'react';
import { useSelector, useDispatch } from 'react-redux';

import Frame from 'react-frame-component';
import { getMessages } from '../../store/actions/message.actions';
import WidgetContext from '../../widget-context';
var LayoutStyleOne = lazy(function () {
  return import('./layout/style-one');
});
var LayoutStyleTwo = lazy(function () {
  return import('./layout/style-two');
});
var LayoutStyleThree = lazy(function () {
  return import('./layout/style-three');
});
var NoConection = lazy(function () {
  return import('./no-connection');
});

export default (function (props) {
  var iframeStyles = props.iframeStyles,
      isInConfigurationMode = props.isInConfigurationMode,
      demoData = props.demoData,
      setting = props.setting,
      isMobileMode = props.isMobileMode,
      area = props.area,
      rect = props.rect,
      isSimulator = props.isSimulator,
      position = props.position,
      isShowChat = props.isShowChat,
      isSettingCollapse = props.isSettingCollapse,
      isSettingLeftBottom = props.isSettingLeftBottom,
      isSettingVertical = props.isSettingVertical,
      marginBottom = props.marginBottom,
      marginRight = props.marginRight,
      marginLeft = props.marginLeft;

  var _useContext = useContext(WidgetContext),
      online = _useContext.online;

  var dispatch = useDispatch();
  var iframeRef = useRef();
  var activeTheme = setting.activeTheme,
      isEnableAutomation = setting.isEnableAutomation,
      isInOperatingHours = setting.isInOperatingHours;

  var isStartConversationFromAdmin = useSelector(function (store) {
    return _get(store, 'behavior.isStartConversationFromAdmin');
  });
  var alreadySubmitUserInfo = useSelector(function (store) {
    return _get(store, 'behavior.alreadySubmitUserInfo');
  });
  var alreadyStartConversation = useSelector(function (store) {
    return _get(store, 'behavior.alreadyStartConversation');
  });
  var hasTriggeredWelcomeMessage = useSelector(function (store) {
    return _get(store, 'behavior.hasTriggeredWelcomeMessage');
  });
  var lastTimeAcceptChattingWithBot = useSelector(function (store) {
    return _get(store, 'behavior.lastTimeAcceptChattingWithBot');
  });
  var lastTimeAcceptLeavingOfflineMessage = useSelector(function (store) {
    return _get(store, 'behavior.lastTimeAcceptLeavingOfflineMessage');
  });
  var lastTimeReceivingMessageFromAdmin = useSelector(function (store) {
    return _get(store, 'behavior.lastTimeReceivingMessageFromAdmin');
  });

  var messages = useSelector(function (store) {
    return _get(store, 'messages.messages');
  });
  var participants = useSelector(function (store) {
    return _get(store, 'messages.participants');
  });
  var showChat = useSelector(function (store) {
    return _get(store, 'behavior.showChat');
  });

  var getFrameStyle = function getFrameStyle() {
    var isLeftSite = _get(setting, 'chatboxComponent.position') === 'left-bottom';
    if (isMobileMode) {
      return {
        right: 0,
        bottom: 0,
        position: 'fixed',
        width: '100%',
        minHeight: '250px',
        maxHeight: '100%',
        maxWidth: 'unset',
        // boxShadow: isShowChat ? 'rgba(0, 0, 0, 0.16) 0px 5px 40px' : 'none', // Phần content load sau gây cảm giác chậm
        opacity: isSimulator ? '1' : isShowChat ? '1' : '0',
        transform: isSimulator ? 'scale(1, 1)' : isShowChat ? 'scale(1, 1)' : 'scale(0, 0)',
        transition: 'transform 0.3s ease-in-out,opacity 0.3s ease-in-out , box-shadow 0.3s ease-in-out',
        height: '100%',
        zIndex: 9999998,
        transformOrigin: 'bottom right'
      };
    } else {
      var windowWidth = rect.width;
      var iconSize = setting.iconSize,
          iconMargin = setting.iconMargin;

      var bottom = 10;
      var right = void 0;
      var width = 380;

      if (isSettingCollapse) {
        if (isSettingLeftBottom) {
          if (isSettingVertical) {
            bottom = marginBottom;
            right = position.right - width - iconMargin;
          } else {
            bottom = marginBottom + 10 + iconSize;
            right = windowWidth - width - marginLeft;
          }
        } else {
          if (isSettingVertical) {
            right = (position.right || 10) + iconSize + iconMargin;
            bottom = marginBottom;
          } else {
            bottom = marginBottom + iconSize + 10;
            right = marginRight;
          }
        }
      } else {
        if (isSettingLeftBottom) {
          if (isSettingVertical) {
            bottom = marginBottom;
            right = position.right - width - iconMargin;
          } else {
            bottom = marginBottom + iconSize + iconMargin;
            right = windowWidth - width - marginLeft;
          }
        } else {
          if (isSettingVertical) {
            bottom = marginBottom;
            right = position.right + iconSize + 10;
          } else {
            bottom = marginBottom + 10 + iconSize;
            right = marginRight;
          }
        }
      }

      return {
        right: right + 'px',
        bottom: bottom + 'px',
        position: 'absolute',
        width: width + 'px',
        minHeight: '250px',
        maxHeight: '600px',
        maxWidth: 'unset',
        // boxShadow: isShowChat ? 'rgba(0, 0, 0, 0.16) 0px 5px 40px' : 'none',
        opacity: isSimulator ? '1' : isShowChat ? '1' : '0',
        height: '600px',
        zIndex: 9999998,
        background: 'transparent',
        transform: isSimulator ? 'scale(1, 1)' : isShowChat ? 'scale(1, 1)' : 'scale(0, 0)',
        transition: 'transform 0.3s ease-in-out,opacity 0.3s ease-in-out , box-shadow 0.3s ease-in-out',
        transformOrigin: isSettingLeftBottom ? 'bottom left' : 'bottom right'
      };
    }
  };

  useEffect(function () {
    if (showChat && !isInConfigurationMode && !isSimulator) {
      dispatch(getMessages(false, setting.channelId));
    }
  }, [showChat]);

  useEffect(function () {
    // console.log('iframeStyles change!');
    if (iframeRef.current) {
      var stylesheetNode = _find(iframeRef.current.node.contentDocument.styleSheets, function (node) {
        return _get(node, 'ownerNode.className') === 'iframe-boostcrm';
      });

      if (stylesheetNode && stylesheetNode.ownerNode) {
        stylesheetNode.ownerNode.outerHTML = iframeStyles;
      }
    }
  }, [iframeStyles]);

  if (!isInConfigurationMode && !online) {
    console.log('!isInConfigurationMode && !online: ', !isInConfigurationMode && !online);
    console.log('isInConfigurationMode: ', isInConfigurationMode);
    console.log('online: ', online);
  }

  return React.createElement(
    Frame,
    {
      ref: iframeRef,
      initialContent: '<!DOCTYPE html><html><head>          \n        <link href="https://fonts.googleapis.com/css?family=Comfortaa&display=swap" rel="stylesheet">\n        <link href="https://fonts.googleapis.com/css?family=Roboto&display=swap" rel="stylesheet">\n        <link href="https://fonts.googleapis.com/css2?family=Open+Sans&display=swap" rel="stylesheet">\n        <style>\n          .bs-chatbox-input-container .ant-upload {\n            display: flex !important;\n          }\n          .bs-chat-box-form .ant-form-item-control {\n            line-height: 31px !important;\n          }\n        </style>\n        ' + iframeStyles + '</head><body></body></html>\n      ',
      mountTarget: 'body',
      allowtransparency: 'yes',
      style: getFrameStyle(),
      frameBorder: '0',
      id: 'bs-chatbox'
    },
    !isInConfigurationMode && !online ? React.createElement(NoConection, _extends({}, props, { isInConfigurationMode: isInConfigurationMode, demoData: demoData, isMobileMode: isMobileMode })) : React.createElement(
      'div',
      { style: { width: '100%', height: '100%' } },
      activeTheme === 'style-one' && React.createElement(
        Suspense,
        { fallback: React.createElement('div', null) },
        React.createElement(LayoutStyleOne, _extends({}, props, {
          lastTimeAcceptChattingWithBot: lastTimeAcceptChattingWithBot,
          lastTimeAcceptLeavingOfflineMessage: lastTimeAcceptLeavingOfflineMessage,
          lastTimeReceivingMessageFromAdmin: lastTimeReceivingMessageFromAdmin,
          isStartConversationFromAdmin: isStartConversationFromAdmin,
          alreadySubmitUserInfo: alreadySubmitUserInfo,
          messages: messages,
          participants: participants,
          isInConfigurationMode: isInConfigurationMode,
          demoData: demoData,
          isEnableAutomation: isEnableAutomation,
          isInOperatingHours: isInOperatingHours
        }))
      ),
      activeTheme === 'style-two' && React.createElement(
        Suspense,
        { fallback: React.createElement('div', null) },
        React.createElement(LayoutStyleTwo, _extends({}, props, {
          lastTimeAcceptChattingWithBot: lastTimeAcceptChattingWithBot,
          lastTimeAcceptLeavingOfflineMessage: lastTimeAcceptLeavingOfflineMessage,
          lastTimeReceivingMessageFromAdmin: lastTimeReceivingMessageFromAdmin,
          isStartConversationFromAdmin: isStartConversationFromAdmin,
          alreadySubmitUserInfo: alreadySubmitUserInfo,
          alreadyStartConversation: alreadyStartConversation,
          messages: messages,
          participants: participants,
          isInConfigurationMode: isInConfigurationMode,
          demoData: demoData,
          isEnableAutomation: isEnableAutomation,
          isInOperatingHours: isInOperatingHours
        }))
      ),
      activeTheme === 'style-three' && React.createElement(
        Suspense,
        { fallback: React.createElement('div', null) },
        React.createElement(LayoutStyleThree, _extends({}, props, {
          lastTimeAcceptChattingWithBot: lastTimeAcceptChattingWithBot,
          lastTimeAcceptLeavingOfflineMessage: lastTimeAcceptLeavingOfflineMessage,
          lastTimeReceivingMessageFromAdmin: lastTimeReceivingMessageFromAdmin,
          isStartConversationFromAdmin: isStartConversationFromAdmin,
          alreadySubmitUserInfo: alreadySubmitUserInfo,
          alreadyStartConversation: alreadyStartConversation,
          hasTriggeredWelcomeMessage: hasTriggeredWelcomeMessage,
          messages: messages,
          participants: participants,
          isInConfigurationMode: isInConfigurationMode,
          demoData: demoData,
          isEnableAutomation: isEnableAutomation,
          isInOperatingHours: isInOperatingHours
        }))
      )
    )
  );
});