import _extends from 'babel-runtime/helpers/extends';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import _size from 'lodash/size';
import _isEmpty from 'lodash/isEmpty';
import _get from 'lodash/get';
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';

import { sendTextMessage, sendAttachment } from '../../../store/actions/message.actions';
import StyleOne from './style-one';
import StyleTwo from './style-two';
import StyleThree from './style-three';
import checkValidFile from './utils/check-valid-file';
import isImageFile from './utils/is-image-file';
import getBase64 from './utils/get-base64';
import getFileMetaData from './utils/get-file-meta-data';
var MAX_COUNT_FILE = 3;

var BSChatBoxInput = function (_Component) {
  _inherits(BSChatBoxInput, _Component);

  function BSChatBoxInput() {
    var _temp, _this, _ret;

    _classCallCheck(this, BSChatBoxInput);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _Component.call.apply(_Component, [this].concat(args))), _this), _this.state = {
      inputValue: '',
      uploadFiles: []
    }, _this.checkInputValueAndSend = function () {
      var _this$state = _this.state,
          inputValue = _this$state.inputValue,
          uploadFiles = _this$state.uploadFiles;
      var setting = _this.props.setting;

      var trimValue = (inputValue || '').trim();
      if (_isEmpty(trimValue) && _isEmpty(uploadFiles)) return;

      if (!_isEmpty(uploadFiles)) {
        _this.props.actions.sendAttachment(inputValue.trim(), uploadFiles, setting.channelId);
        _this.setState({
          inputValue: '',
          uploadFiles: []
        });
        return;
      }

      if (!_isEmpty(inputValue)) {
        _this.props.actions.sendTextMessage(inputValue.trim(), setting.channelId);
        _this.setState({
          inputValue: ''
        });
        return;
      }
    }, _this.handleKeyPress = function (event) {
      // console.log(`handleKeyPress ${event.key} - ${event.shiftKey}`);
      if (event.key === 'Enter' && !event.shiftKey) {
        _this.checkInputValueAndSend();
        event.preventDefault();
      }
    }, _this.handleOnChange = function (event) {
      _this.setState({
        inputValue: event.target.value
      });
    }, _this.handleClickBtnSend = function () {
      _this.checkInputValueAndSend();
    }, _this.handleBeforeUploadAttachments = function (file) {
      var uploadFiles = _this.state.uploadFiles;
      // console.log("file", file);

      if (!checkValidFile(file) || _size(uploadFiles) >= MAX_COUNT_FILE) {
        window && window.alert('Max 3 files, max size 1MB');
        return false;
      }

      var isImage = isImageFile(file);
      var newAttachment = _extends({
        originalFile: file
      }, getFileMetaData(file), {
        isImage: isImage
      });

      isImage ? getBase64(file, function (base64) {
        _this.setState({
          uploadFiles: [].concat(uploadFiles, [_extends({}, newAttachment, {
            base64: base64
          })])
        });
      }) : _this.setState({
        uploadFiles: [].concat(uploadFiles, [newAttachment])
      });

      return false;
    }, _this.handleRemoveAttachment = function (attachment) {
      _this.setState({
        uploadFiles: _this.state.uploadFiles.filter(function (obj) {
          return obj !== attachment;
        })
      });
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  BSChatBoxInput.prototype.render = function render() {
    var callbacks = {
      handleKeyPress: this.handleKeyPress.bind(this),
      handleOnChange: this.handleOnChange.bind(this),
      handleClickBtnSend: this.handleClickBtnSend.bind(this),
      handleRemoveAttachment: this.handleRemoveAttachment.bind(this),
      handleBeforeUploadAttachments: this.handleBeforeUploadAttachments.bind(this)
    };
    var setting = this.props.setting;

    var activeTheme = _get(setting, 'activeTheme');
    // console.log("checkporps", this.props);
    return React.createElement(
      'div',
      { style: { height: '80px', flexGrow: '0' } },
      activeTheme === 'style-two' && React.createElement(StyleTwo, _extends({}, this.props, {
        attachments: this.state.attachments
      }, callbacks, {
        uploadFiles: this.state.uploadFiles,
        inputValue: this.state.inputValue
      })),
      activeTheme === 'style-one' && React.createElement(StyleOne, _extends({}, this.props, {
        attachments: this.state.attachments
      }, callbacks, {
        uploadFiles: this.state.uploadFiles,
        inputValue: this.state.inputValue
      })),
      activeTheme === 'style-three' && React.createElement(StyleThree, _extends({}, this.props, {
        attachments: this.state.attachments
      }, callbacks, {
        uploadFiles: this.state.uploadFiles,
        inputValue: this.state.inputValue
      }))
    );
  };

  return BSChatBoxInput;
}(Component);

export default connect(function (store) {
  return {};
}, function (dispatch) {
  return {
    actions: bindActionCreators({
      sendTextMessage: sendTextMessage,
      sendAttachment: sendAttachment
    }, dispatch)
  };
})(BSChatBoxInput);