import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import _last from 'lodash/last';
import _throttle from 'lodash/throttle';
import _delay from 'lodash/delay';
import _size from 'lodash/size';
import _get from 'lodash/get';
import React from 'react';
import MessageContainer from 'chat-message-container';

import Sound from 'react-sound';
import * as sound from '../../sounds/sound';
import { getMessages, onScrollToBottom } from '../../../store/actions';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';

var ChatMessageContainer = function (_React$Component) {
  _inherits(ChatMessageContainer, _React$Component);

  function ChatMessageContainer(props) {
    _classCallCheck(this, ChatMessageContainer);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _this.scrollToBottom = function () {
      if (_this.refs.chatMessageContainer) {
        _delay(function () {
          _this.refs.chatMessageContainer && _this.refs.chatMessageContainer.scrollToBottom();
          _delay(function () {
            _this.props.actions.onScrollToBottom();
          }, 500);
        }, 100);
      }
    };

    _this.isScrollingOnBottom = function () {
      if (_this.refs.chatMessageContainer) {
        return _this.refs.chatMessageContainer.isAtBottom();
      }
      return false;
    };

    _this.handleSoundStartFinishedPlaying = function (value) {
      _this.setState({ isNewMessage: value });
    };

    _this.loadMoreMessages = function () {
      var _this$props = _this.props,
          isLoadingMessage = _this$props.isLoadingMessage,
          setting = _this$props.setting;

      if (isLoadingMessage) return;
      _this.throttledLoadMessageHistory(setting.channelId);
    };

    _this.isScrollingOnTop = function () {
      if (_this.refs.chatMessageContainer) {
        var _this$refs$chatMessag = _this.refs.chatMessageContainer.scrollbarRef.getValues(),
            top = _this$refs$chatMessag.top;

        if (top === 0) {
          return true;
        }
      }
      return false;
    };

    _this.scrollToPositionBeforeLoadMore = function () {
      var messageContainerHeight = _this.state.messageContainerHeight;

      if (_this.refs.chatMessageContainer) {
        var currentHeight = _this.refs.chatMessageContainer.scrollbarRef.getScrollHeight();
        _delay(function () {
          var scrollTopValue = currentHeight - messageContainerHeight;
          _this.refs.chatMessageContainer.scrollbarRef.scrollTop(scrollTopValue);
        }, 100);
      }
    };

    _this.updateMessageContainerHeight = function () {
      _this.setState({
        messageContainerHeight: _this.refs.chatMessageContainer.scrollbarRef.getScrollHeight()
      });
    };

    _this.state = {
      isNewMessage: false,
      originalMessage: null
    };
    _this.throttledLoadMessageHistory = _throttle(function (channelId) {
      _this.props.actions.getMessages(true, channelId);
    }, 500, { trailing: false });
    _this.throttledPlaySound = _throttle(function (value) {
      _this.handleSoundStartFinishedPlaying(value);
    }, 2000, { trailing: false });
    return _this;
  }

  ChatMessageContainer.prototype.componentDidMount = function componentDidMount() {
    //Scroll to bottom when F5 page
    this.scrollToBottom();
  };

  ChatMessageContainer.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps, prevState) {
    if (nextProps.messages && nextProps.messages !== prevState.originalMessage) {
      return {
        originalMessage: nextProps.messages
      };
    } else {
      return null;
    }
  };

  ChatMessageContainer.prototype.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    var sender = _get(_last(this.state.originalMessage), 'senderId');
    if (prevState.originalMessage !== this.state.originalMessage) {
      if (_size(this.state.originalMessage) - _size(prevState.originalMessage) === 1) {
        if (sender === 'subscriber') {
          this.scrollToBottom();
        } else {
          //playsoud when receiving new message
          if (this.props.soundOn && this.props.numberOfNewMessages !== 0) {
            this.throttledPlaySound(true);
          }
          //is at bottom
          if (this.isScrollingOnBottom()) {
            this.scrollToBottom();
          }
        }
      }
      //loadmore message
      if (_size(prevState.originalMessage) !== 0 && _size(prevState.originalMessage) < _size(this.state.originalMessage) && this.isScrollingOnTop()) {
        this.scrollToPositionBeforeLoadMore();
      }
      //update scroll height
      this.updateMessageContainerHeight();
    }
  };

  ChatMessageContainer.prototype.render = function render() {
    var _this2 = this;

    var _props = this.props,
        containerBgColor = _props.containerBgColor,
        isShowLeftAvatar = _props.isShowLeftAvatar,
        isShowRightAvatar = _props.isShowRightAvatar,
        isShowSeenTime = _props.isShowSeenTime,
        avatarWidth = _props.avatarWidth,
        avatarMargin = _props.avatarMargin,
        style = _props.style,
        textFontSize = _props.textFontSize,
        messageMargin = _props.messageMargin,
        onClickButton = _props.onClickButton,
        leftParitcipantsTextColor = _props.leftParitcipantsTextColor,
        leftParitcipantsTextBgColor = _props.leftParitcipantsTextBgColor,
        rightParitcipantsTextColor = _props.rightParitcipantsTextColor,
        rightParitcipantsTextBgColor = _props.rightParitcipantsTextBgColor,
        messages = _props.messages,
        participants = _props.participants;
    // console.log("message ", messages);

    return React.createElement(
      React.Fragment,
      null,
      this.state.isNewMessage && React.createElement(Sound, {
        url: sound.SOUND_NEW_MESSAGE,
        playStatus: Sound.status.PLAYING,
        onFinishedPlaying: function onFinishedPlaying() {
          return _this2.handleSoundStartFinishedPlaying(false);
        }
      }),
      React.createElement(MessageContainer, {
        containerBgColor: containerBgColor,
        isShowLeftAvatar: isShowLeftAvatar,
        isShowRightAvatar: isShowRightAvatar,
        isShowSeenTime: isShowSeenTime,
        avatarWidth: avatarWidth,
        avatarMargin: avatarMargin,
        style: style,
        textFontSize: textFontSize,
        messageMargin: messageMargin,
        onScrollTop: function onScrollTop() {
          return _this2.loadMoreMessages();
        },
        onScrollBottom: function onScrollBottom() {
          return _this2.scrollToBottom();
        },
        onClickButton: onClickButton,
        leftParitcipantsTextColor: leftParitcipantsTextColor,
        leftParitcipantsTextBgColor: leftParitcipantsTextBgColor,
        rightParitcipantsTextColor: rightParitcipantsTextColor,
        rightParitcipantsTextBgColor: rightParitcipantsTextBgColor,
        messages: messages,
        participants: participants,
        noMessageText: 'No messages',
        isChatbox: true,
        ref: 'chatMessageContainer'
      })
    );
  };

  return ChatMessageContainer;
}(React.Component);

export default connect(function (store) {
  return { numberOfNewMessages: _get(store, 'messages.numberOfNewMessages') };
}, function (dispatch) {
  return {
    actions: bindActionCreators({
      getMessages: getMessages,
      onScrollToBottom: onScrollToBottom
    }, dispatch)
  };
})(ChatMessageContainer);