import _get from 'lodash/get';
import _inCludes from 'lodash/includes';
import React, { Component, useContext } from 'react';
import commonStyles from '../../common-styles.scss';
import UserInfoForm from '../../user-info-form';
import Header from '../../header';
import Power from '../../power';
import ChatBoxInput from '../../input';
import { getCurrentScreen } from '../../../../commons/utils';
import OfflineStatus from '../../offline';
import { useDispatch, useSelector } from 'react-redux';
import { sendButtonMessage } from '../../../../store/actions';
import ChatMessageContainer from '../message-container';
import WidgetContext from '../../../../widget-context';

import NoConect from '../../no-connection';

export default (function (props) {
  var alreadyStartConversation = props.alreadyStartConversation,
      isInConfigurationMode = props.isInConfigurationMode,
      isMobileMode = props.isMobileMode,
      setting = props.setting,
      messages = props.messages,
      participants = props.participants,
      demoData = props.demoData;

  var dispatch = useDispatch();

  var _useContext = useContext(WidgetContext),
      isSimulator = _useContext.isSimulator;

  var showedScreen = isSimulator ? 'conversation' : getCurrentScreen(props);
  var soundOn = isSimulator ? false : useSelector(function (store) {
    return _get(store, 'behavior.soundOn');
  });
  var isViewForOffline = showedScreen === 'off' || showedScreen === 'off_with_chatbot';

  return React.createElement(
    'div',
    {
      className: commonStyles.conversationContainer,
      style: {
        borderRadius: '' + (isMobileMode ? '0' : '18px')
      }
    },
    isViewForOffline && React.createElement(OfflineStatus, { setting: setting, showedScreen: showedScreen }),
    !isViewForOffline && React.createElement(
      React.Fragment,
      null,
      !isSimulator && !_inCludes(['off', 'no_connection'], showedScreen) && React.createElement(Header, {
        setting: setting,
        isInConfigurationMode: isInConfigurationMode,
        demoData: demoData,
        currentScreen: showedScreen,
        isMobileMode: isMobileMode
      }),
      React.createElement(
        'div',
        { className: commonStyles.boxBody },
        showedScreen === 'form' && React.createElement(UserInfoForm, { setting: setting, isInConfigurationMode: isInConfigurationMode }),
        showedScreen === 'conversation' && React.createElement(
          'div',
          { className: commonStyles.messageContainer },
          React.createElement(ChatMessageContainer, {
            containerBgColor: 'white',
            isShowLeftAvatar: false,
            isShowRightAvatar: false,
            isShowSeenTime: false,
            avatarWidth: 25,
            avatarMargin: 0,
            style: 2,
            textFontSize: 12,
            messageMargin: 25,
            onScrollTop: function onScrollTop() {},
            onClickButton: function onClickButton(buttonData) {
              return dispatch(sendButtonMessage(buttonData, setting.channelId));
            },
            leftParitcipantsTextColor: _get(setting, 'screenConversation.leftSideTextColor'),
            leftParitcipantsTextBgColor: _get(setting, 'screenConversation.leftSideBackgroundColor'),
            rightParitcipantsTextColor: _get(setting, 'screenConversation.rightSideTextColor'),
            rightParitcipantsTextBgColor: _get(setting, 'screenConversation.rightSideBackgroundColor'),
            messages: isInConfigurationMode ? demoData.messages : messages,
            participants: isInConfigurationMode ? demoData.participants : participants,
            soundOn: soundOn,
            setting: setting
          })
        ),
        showedScreen === 'conversation' && React.createElement(ChatBoxInput, { setting: setting, isInConfigurationMode: isInConfigurationMode }),
        showedScreen === 'no_connection' && React.createElement(NoConect, { setting: setting, isInConfigurationMode: isInConfigurationMode, demoData: demoData, isMobileMode: isMobileMode })
      )
    ),
    React.createElement(Power, { setting: setting, showedScreen: showedScreen, isInConfigurationMode: isInConfigurationMode })
  );
});