import _get from 'lodash/get';
import React, { useContext } from 'react';
import Header from '../../header';
import Power from '../../power';
import commonStyles from '../../common-styles.scss';
import noConnectionSVG from '../../../icon/no_connection.svg';
import SVG from 'react-inlinesvg';

import WidgetContext from '../../../../widget-context';
import { clickReloadHandler } from '../common-handlers';
import reloadSVG from '../../../icon/reload.svg';

export default (function (props) {
  var setting = props.setting,
      isInConfigurationMode = props.isInConfigurationMode,
      demoData = props.demoData,
      showedScreen = props.showedScreen;

  var _useContext = useContext(WidgetContext),
      isReconnecting = _useContext.isReconnecting,
      isConnectFailed = _useContext.isConnectFailed;

  var isShowReloadContent = !isReconnecting && isConnectFailed || isInConfigurationMode;
  return React.createElement(
    'div',
    { className: commonStyles.noConnectionContainer },
    React.createElement(Header, { setting: setting, isInConfigurationMode: isInConfigurationMode, demoData: demoData, currentScreen: showedScreen }),
    React.createElement(
      'div',
      { className: commonStyles.boxBody },
      React.createElement(SVG, { className: commonStyles.image, src: noConnectionSVG }),
      isReconnecting && !isConnectFailed && React.createElement(
        'span',
        { className: commonStyles.wsStatus },
        'Reconnecting...'
      ),
      isShowReloadContent && React.createElement(
        React.Fragment,
        null,
        React.createElement(
          'span',
          { className: commonStyles.title },
          _get(setting, 'chatboxThemeSettings.commonSettings.no_conect_title')
        ),
        React.createElement(
          'span',
          { className: commonStyles.subtitle },
          _get(setting, 'chatboxThemeSettings.commonSettings.no_conect_subtitle')
        ),
        React.createElement(
          'span',
          {
            className: commonStyles.btnReload,
            onClick: clickReloadHandler,
            style: {
              color: _get(setting, 'screenConversation.rightSideTextColor'),
              backgroundColor: _get(setting, 'screenConversation.rightSideBackgroundColor')
            }
          },
          React.createElement(SVG, { className: commonStyles.reloadIcon, src: reloadSVG }),
          _get(setting, 'chatboxThemeSettings.commonSettings.no_conect_btn_reload_label') || 'Reload'
        )
      )
    ),
    React.createElement(Power, { setting: setting, showedScreen: showedScreen })
  );
});