import _get from 'lodash/get';
import _isEmpty from 'lodash/isEmpty';
import _includes from 'lodash/includes';
import React, { useState } from 'react';
import styles from './user-info-form.scss';
import { startConversation } from '../../../store/actions';
import BSChatBoxFormInput from '../../../commons/components/form-input';
import BSChatBoxFormCheckBox from '../../../commons/components/form-checkbox';
import StartButton from './btn-start-conversation';

import SVGSkipForm from '../../icon/skip-form.svg';
import SVG from 'react-inlinesvg';
import SVGName from '../../icon/name.svg';
import SVGEmail from '../../icon/email.svg';
import SVGPhone from '../../icon/phone.svg';
import { useForm } from 'react-hook-form';
import { useDispatch } from 'react-redux';
import { BUTTON_STARTS } from '../../../commons/constants/websocket';

export default (function (_ref) {
  var setting = _ref.setting;

  var _useState = useState(null),
      name = _useState[0],
      setName = _useState[1];

  var _useState2 = useState(null),
      email = _useState2[0],
      setEmail = _useState2[1];

  var _useState3 = useState(null),
      phone = _useState3[0],
      setPhone = _useState3[1];

  var _useState4 = useState(false),
      checkPrivacyPolicyConsent = _useState4[0],
      setCheckPrivacyPolicyConsent = _useState4[1];

  var _useForm = useForm(),
      handleSubmit = _useForm.handleSubmit;

  var _useState5 = useState(false),
      isSubmit = _useState5[0],
      setSubmit = _useState5[1];

  var dispatch = useDispatch();

  var activeTheme = _get(setting, 'activeTheme');
  var enableFieldName = _get(setting, 'chatboxThemeSettings.commonSettings.form_name_enable');
  var enableFieldEmail = _get(setting, 'chatboxThemeSettings.commonSettings.form_email_enable');
  var enableFieldPhone = _get(setting, 'chatboxThemeSettings.commonSettings.form_phone_enable');
  var enablePrivacyPolicyConsent = _get(setting, 'chatboxThemeSettings.preChatForm.privacy_consent_enable') === true;

  var privacyPolicyMessage = _get(setting, 'chatboxThemeSettings.preChatForm.privacy_consent_message');
  var privacyPolicyLink = _get(setting, 'chatboxThemeSettings.preChatForm.privacy_consent_link');
  var requiredFieldName = _get(setting, 'chatboxThemeSettings.commonSettings.form_name_require');
  var requiredFieldEmail = _get(setting, 'chatboxThemeSettings.commonSettings.form_email_require');
  var requiredFieldPhone = _get(setting, 'chatboxThemeSettings.commonSettings.form_phone_require');
  var isInOperatingHours = _get(setting, 'isInOperatingHours');
  var canSkip = !requiredFieldName && !requiredFieldPhone && !requiredFieldEmail;

  var onSubmit = function onSubmit() {
    var validateName = enableFieldName && requiredFieldName && _isEmpty(name) ? false : true;
    var validateEmail = enableFieldEmail && requiredFieldEmail && _isEmpty(email) ? false : true;
    var validatePhone = enableFieldPhone && requiredFieldPhone && _isEmpty(phone) ? false : true;
    setSubmit(true);

    if (enablePrivacyPolicyConsent && !checkPrivacyPolicyConsent) return;
    if (!_includes([validateName, validatePhone, validateEmail], false)) {
      dispatch(startConversation(BUTTON_STARTS.START_CONVERSATION, { name: name, email: email, phone: phone, isWorkingDay: setting.isInOperatingHours }));
      return;
    }

    if (canSkip) {
      dispatch(startConversation(BUTTON_STARTS.START_CONVERSATION, { name: name, email: email, phone: phone, isWorkingDay: setting.isInOperatingHours }));
    }
  };

  var getInputTextColor = function getInputTextColor(setting) {
    var activeTheme = _get(setting, 'activeTheme');
    if (activeTheme === 'style-one') {
      return _get(setting, 'screenForm.inputTextColor');
    } else if (activeTheme === 'style-two') {
      return _get(setting, 'screenConversation.rightSideBackgroundColor');
    } else {
      return _get(setting, 'screenConversation.rightSideBackgroundColor');
    }
  };

  // console.log('check: ', checkPrivacyPolicyConsent)
  return React.createElement(
    'div',
    { className: styles.userInfoFormContainer + ' bs-chat-box-form' },
    React.createElement(
      'div',
      { className: styles.form },
      React.createElement(
        'form',
        { onSubmit: handleSubmit(onSubmit), className: styles.userForm },
        React.createElement(
          'div',
          null,
          enableFieldName && React.createElement(BSChatBoxFormInput, {
            style: activeTheme,
            backgroundColor: _get(setting, 'screenForm.inputBackgroundColor'),
            isRequired: requiredFieldName,
            textColor: getInputTextColor(setting),
            icon: React.createElement(SVG, { src: SVGName, style: { width: '12px', height: '12px' } }),
            placeholder: _get(setting, 'chatboxThemeSettings.commonSettings.form_name_placeholder'),
            onChange: function onChange(e) {
              return setName(e.target.value);
            },
            isValid: isSubmit ? _isEmpty(name) : undefined,
            inputValue: name,
            placeholderColor: _get(setting, 'screenForm.placeholderTextColor')
          })
        ),
        React.createElement(
          'div',
          null,
          enableFieldEmail && React.createElement(BSChatBoxFormInput, {
            style: activeTheme,
            backgroundColor: _get(setting, 'screenForm.inputBackgroundColor'),
            isRequired: requiredFieldEmail,
            textColor: getInputTextColor(setting),
            icon: React.createElement(SVG, { src: SVGEmail, style: { width: '16px', height: '11px' } }),
            placeholder: _get(setting, 'chatboxThemeSettings.commonSettings.form_email_placeholder'),
            onChange: function onChange(e) {
              return setEmail(e.target.value);
            },
            isValid: isSubmit ? _isEmpty(email) : undefined,
            inputValue: email,
            placeholderColor: _get(setting, 'screenForm.placeholderTextColor')
          })
        ),
        React.createElement(
          'div',
          null,
          enableFieldPhone && React.createElement(BSChatBoxFormInput, {
            style: activeTheme,
            backgroundColor: _get(setting, 'screenForm.inputBackgroundColor'),
            isRequired: requiredFieldPhone,
            textColor: getInputTextColor(setting),
            icon: React.createElement(SVG, { style: { width: '16.8px', height: '16.8px' }, src: SVGPhone }),
            placeholder: _get(setting, 'chatboxThemeSettings.commonSettings.form_phone_placeholder'),
            onChange: function onChange(e) {
              return setPhone(e.target.value);
            },
            isValid: isSubmit ? _isEmpty(phone) : undefined,
            inputValue: phone,
            placeholderColor: _get(setting, 'screenForm.placeholderTextColor')
          })
        ),
        React.createElement(
          'div',
          null,
          enablePrivacyPolicyConsent && React.createElement(BSChatBoxFormCheckBox, {
            color: isSubmit && !checkPrivacyPolicyConsent ? 'red' : _get(setting, 'screenConversation.rightSideBackgroundColor'),
            label: privacyPolicyMessage,
            link: privacyPolicyLink,
            checked: checkPrivacyPolicyConsent,
            onChange: function onChange(checked) {
              return setCheckPrivacyPolicyConsent(checked);
            }
          })
        ),
        React.createElement(
          'div',
          null,
          React.createElement(StartButton, { setting: setting })
        ),
        React.createElement(
          'div',
          null,
          canSkip && React.createElement(
            'div',
            {
              onClick: onSubmit,
              className: styles.skipWrapper,
              style: {
                color: _get(setting, 'screenConversation.rightSideBackgroundColor')
              }
            },
            React.createElement(
              'span',
              null,
              _get(setting, 'chatboxThemeSettings.commonSettings.skip_button_label')
            ),
            React.createElement(
              'span',
              null,
              React.createElement(SVG, {
                src: SVGSkipForm,
                style: {
                  width: '7.5px',
                  height: '4.8px',
                  marginLeft: '3.8px'
                }
              })
            )
          )
        )
      )
    )
  );
});