import _extends from 'babel-runtime/helpers/extends';
import _throttle from 'lodash/throttle';
import _get from 'lodash/get';
import React, { Component, useCallback, useState } from 'react';
import { connect, useDispatch } from 'react-redux';
import { bindActionCreators } from 'redux';

import Frame from 'react-frame-component';
import { toggleChat } from '../../store/actions';
import chatIcon from '../icon/live-chat.svg';
import WidgetIcon from '../../commons/components/widget-icon';

import Tooltip from '../tooltip';
export default (function (_ref) {
  var numberOfNewMessages = _ref.numberOfNewMessages,
      props = _ref.props;
  var iframeStyles = props.iframeStyles,
      position = props.position,
      setting = props.setting,
      svgSize = props.svgSize;

  var _useState = useState(false),
      isHover = _useState[0],
      setIsHover = _useState[1];

  var iconBackgroundColor = setting.iconBackgroundColor,
      iconTextColor = setting.iconTextColor;

  var iconSize = _get(setting, 'iconSize');
  var dispatch = useDispatch();
  var throttleOnClickIcon = useCallback(_throttle(function () {
    dispatch(toggleChat(setting));
  }, 300, { trailing: false }), []);

  return React.createElement(
    'div',
    null,
    React.createElement(
      Frame,
      {
        initialContent: '<!DOCTYPE html><html><head>' + iframeStyles + '</head><body></body></html>',
        mountTarget: 'body',
        style: {
          width: iconSize,
          height: iconSize,
          position: 'absolute',
          bottom: position.bottom + 'px',
          right: position.right + 'px',
          transition: 'opacity 0.8s',
          zIndex: '9999997',
          maxWidth: iconSize,
          minWidth: iconSize,
          minHeight: iconSize,
          maxHeight: iconSize
        },
        allowtransparency: 'yes',
        frameBorder: '0'
      },
      React.createElement(WidgetIcon, {
        iconBackgroundColor: iconBackgroundColor,
        iconSize: iconSize,
        iconSrc: chatIcon,
        iconTextColor: iconTextColor,
        svgSize: svgSize,
        onClick: throttleOnClickIcon,
        numberOfNewMessages: numberOfNewMessages,
        handleHover: function handleHover(e) {
          return setIsHover(e.isHover);
        }
      })
    ),
    isHover && React.createElement(Tooltip, _extends({}, props, { type: 'chat' }))
  );
});