import _get from 'lodash/get';
import React, { Component, useState } from 'react';
import styles from './styles.scss';
import Power from '../power';
import SVG from 'react-inlinesvg';

import SVGEndCall from '../../../icon/call-ended.svg';
import StarRatings from 'react-star-ratings';
import Button from '../button';
import { CloseCallIcon } from '../../../icon';
import { useDispatch } from 'react-redux';
import { updateCallInfo } from '../../../../store/actions/update-call.action';
import { CALL_STATUS } from '../../constants';
import Avatar from '../../../../commons/components/avatar';
import { CALLING_ICON } from '../../../icon';
import CountTime from '../../../../commons/components/count-time';

export default (function (props) {
  var dispatch = useDispatch();
  var setting = props.setting,
      callInfo = props.callInfo,
      status = props.status,
      isMobileMode = props.isMobileMode;

  var _useState = useState(0),
      rating = _useState[0],
      setRating = _useState[1];

  var ratingColor = _get(setting, 'screenConversation.rightSideBackgroundColor');
  var totalDuration = _get(callInfo, 'callData.totalDuration');
  var adminAvatar = _get(callInfo, ['callData', 'admin', 'avatar']);

  var minute = 0;
  var second = 0;
  if (totalDuration) {
    minute = Math.floor(totalDuration / 60);
    second = totalDuration % 60;
  }

  var onChangeRating = function onChangeRating(newRating) {
    var setting = props.setting;

    setRating(newRating);
    dispatch(updateCallInfo({ id: callInfo.callData.id, rating: newRating }, CALL_STATUS.COMPLETED_RATING, setting.channelId));
  };

  var onClickMakeAnotherCall = function onClickMakeAnotherCall() {
    var handleMakeAnotherCall = props.handleMakeAnotherCall;

    handleMakeAnotherCall && handleMakeAnotherCall();
  };
  var isCustomAdminInfo = _get(setting, 'callThemeSettings.isCustomAdminInfo');
  return React.createElement(
    'div',
    { className: styles.container },
    status === CALL_STATUS.COMPLETED_RATING || rating !== 0 ? React.createElement(
      'div',
      { className: styles.ratingResult },
      React.createElement(StarRatings, {
        rating: rating || _get(props.callInfo, ['callData', 'rating']),
        starRatedColor: ratingColor,
        starHoverColor: ratingColor,
        numberOfStars: 5,
        name: 'rating',
        starDimension: '28.8px',
        starSpacing: '7.7px'
      }),
      React.createElement(
        'span',
        { className: styles.title },
        _get(setting, 'callThemeSettings.commonSettings.rating_text')
      ),
      React.createElement(
        'div',
        { className: styles.button },
        React.createElement(Button, {
          setting: setting,
          text: _get(setting, 'callThemeSettings.commonSettings.make_another_call_label'),
          callIcon: true,
          onClick: onClickMakeAnotherCall
        })
      )
    ) : React.createElement(
      'div',
      { className: styles.content },
      React.createElement(
        'div',
        { className: styles.mainContain },
        React.createElement(
          'div',
          { className: styles.endtimeContent },
          React.createElement(
            'div',
            { className: styles.icon },
            React.createElement(SVG, { src: SVGEndCall })
          ),
          React.createElement(
            'div',
            { className: styles.time, style: { fontFamily: 'Open Sans' } },
            React.createElement(CountTime, { minute: minute, second: second }),
            React.createElement(
              'span',
              { style: { marginLeft: '5px' } },
              ' Call ended '
            )
          )
        ),
        React.createElement(
          'div',
          null,
          React.createElement(
            'div',
            { className: styles.userAvatar },
            React.createElement('div', { style: { backgroundColor: '#d6d6d6' }, className: styles.wrapOpacity1 }),
            React.createElement('div', { style: { backgroundColor: '#d6d6d6' }, className: styles.wrapOpacity2 }),
            !isCustomAdminInfo ? React.createElement(
              'div',
              { style: { backgroundColor: '#d6d6d6' }, className: styles.wrap },
              React.createElement(
                'div',
                { className: styles.avartar },
                adminAvatar ? React.createElement(
                  'div',
                  { style: { width: '60px', height: '60px' } },
                  React.createElement(Avatar, { size: '60px', src: adminAvatar })
                ) : React.createElement(Avatar, { size: '30px', src: CALLING_ICON.CALLING3 })
              )
            ) : React.createElement(
              'div',
              { style: { backgroundColor: _get(setting, 'iconTextColor') }, className: styles.wrap },
              React.createElement(
                'div',
                { className: styles.avartar },
                React.createElement(Avatar, { size: '60px', src: _get(setting, 'callThemeSettings.informationSettings.callee_avatar') })
              )
            )
          ),
          React.createElement(
            'div',
            { className: styles.ratingContent },
            React.createElement(
              'span',
              { className: styles.text },
              _get(setting, 'callThemeSettings.commonSettings.pre_rating_text')
            ),
            React.createElement(StarRatings, {
              rating: rating,
              starRatedColor: ratingColor,
              starHoverColor: ratingColor,
              changeRating: function changeRating(newRating, name) {
                return onChangeRating(newRating);
              } // setRating(newRating)
              , numberOfStars: 5,
              name: 'rating',
              starDimension: '27.5px',
              starSpacing: '7.7px'
            })
          )
        )
      ),
      React.createElement(Power, { setting: setting })
    ),
    status === CALL_STATUS.COMPLETED_RATING || rating !== 0 && React.createElement(Power, { setting: setting }),
    React.createElement(CloseCallIcon, { iconTextColor: _get(setting, 'screenConversation.rightSideBackgroundColor'), className: styles.closeIcon })
  );
});