import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import _get from 'lodash/get';
import React, { Component } from 'react';
import styles from './styles.scss';
import { CALL_STATUS, CALL_STOP_REASON } from '../../constants';

import { CloseCallIcon } from '../../../icon';
import Power from '../power';
import SVG from 'react-inlinesvg';
import endCall from '../../../icon/end-call.svg';
import Avatar from '../../../../commons/components/avatar';
import { CALLING_ICON } from '../../../icon';

var PageInitCall = function (_Component) {
  _inherits(PageInitCall, _Component);

  function PageInitCall(props) {
    _classCallCheck(this, PageInitCall);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _this.onClickStop = function () {
      var handleStopCall = _this.props.handleStopCall;
      //ToDo: Server is based on TotalDuration. Need to refactor

      handleStopCall && handleStopCall(CALL_STOP_REASON.SUBSCRIBER_STOP, 0);
      clearTimeout(_this.setTimeoutStopCall);
    };

    return _this;
  }

  PageInitCall.prototype.componentDidMount = function componentDidMount() {
    var _this2 = this;

    // Auto stop call after init 30s but nobody accepting this call
    // console.log("handleAutoStopCall");
    this.setTimeoutStopCall = setTimeout(function () {
      _this2.props.handleStopCall(CALL_STOP_REASON.SUBSCRIBER_AUTO_STOP_WHEN_TIMEOUT);
    }, 60000);
    window.parent.onbeforeunload = function () {
      return 'Do not reload page';
    };
    window.parent.addEventListener('unload', this.onClickStop);
  };

  PageInitCall.prototype.componentWillUnmount = function componentWillUnmount() {
    this.setTimeoutStopCall && clearInterval(this.setTimeoutStopCall);
    window.parent.onbeforeunload = null;
    window.parent.removeEventListener('unload', this.onClickStop);
  };

  PageInitCall.prototype.render = function render() {
    var _props = this.props,
        callInfo = _props.callInfo,
        setting = _props.setting,
        isMobileMode = _props.isMobileMode;

    var status = _get(callInfo, ['callData', 'status']);
    var subscriberName = _get(callInfo.callData, ['callInfo', 'name']);
    var isCustomAvatar = _get(setting, 'callThemeSettings.isCustomProjectAvatar');
    return React.createElement(
      'div',
      { className: styles.container },
      status === CALL_STATUS.INITIATED && React.createElement(
        'div',
        { className: styles.callStatus },
        'Initializing...'
      ),
      status === CALL_STATUS.RINGING && React.createElement(
        'div',
        { className: styles.callStatus },
        'Ringing...'
      ),
      React.createElement(
        'div',
        { className: styles.infoContent },
        React.createElement(
          'div',
          { className: styles.userAvatar },
          React.createElement('div', {
            style: {
              backgroundColor: _get(setting, 'screenConversation.rightSideBackgroundColor'),
              color: _get(setting, 'screenConversation.rightSideBackgroundColor')
            },
            className: styles.wrapOpacity1
          }),
          React.createElement('div', { style: { backgroundColor: _get(setting, 'screenConversation.rightSideBackgroundColor') }, className: styles.wrapOpacity2 }),
          !isCustomAvatar ? React.createElement(
            'div',
            { style: { backgroundColor: _get(setting, 'screenConversation.rightSideBackgroundColor') }, className: styles.wrap },
            React.createElement(
              'div',
              { className: styles.icon },
              React.createElement('img', { src: CALLING_ICON.CALLING1 })
            ),
            React.createElement(
              'div',
              { className: styles.icon },
              React.createElement('img', { src: CALLING_ICON.CALLING2 })
            ),
            React.createElement(
              'div',
              { className: styles.icon },
              React.createElement('img', { src: CALLING_ICON.CALLING3 })
            )
          ) : React.createElement(
            'div',
            { style: { backgroundColor: _get(setting, 'screenConversation.rightSideBackgroundColor') }, className: styles.wrap },
            React.createElement(Avatar, { size: '60px', src: _get(setting, 'callThemeSettings.informationSettings.project_avatar') })
          )
        ),
        React.createElement(
          'div',
          { style: { marginTop: '9px' } },
          React.createElement(
            'span',
            { className: styles.userName },
            _get(setting, 'callThemeSettings.informationSettings.project_name')
          )
        )
      ),
      React.createElement(
        'div',
        { className: styles.endButton, onClick: this.onClickStop },
        React.createElement(
          'div',
          { className: styles.iconContainer },
          React.createElement(SVG, { src: endCall })
        )
      ),
      React.createElement(
        'div',
        { className: styles.power },
        React.createElement(Power, { setting: setting })
      ),
      React.createElement(CloseCallIcon, { className: styles.closeIcon })
    );
  };

  return PageInitCall;
}(Component);

export { PageInitCall as default };