import _get from 'lodash/get';
import _isEmpty from 'lodash/isEmpty';
import _includes from 'lodash/includes';
import React, { Component, useState } from 'react';
import styles from './styles.scss';
import FormInput from '../../../../commons/components/form-input';

import Button from '../button';
import { leaveInformation } from '../../../../store/actions/leave-information.action';

import SVGName from '../../../icon/name.svg';
import SVGEmail from '../../../icon/email.svg';
import SVGPhone from '../../../icon/phone.svg';
import SVG from 'react-inlinesvg';
import { useForm } from 'react-hook-form';
import { useDispatch } from 'react-redux';
import Power from '../power';

export default (function (_ref) {
  var setting = _ref.setting;

  var _useState = useState(null),
      name = _useState[0],
      setName = _useState[1];

  var _useState2 = useState(null),
      email = _useState2[0],
      setEmail = _useState2[1];

  var _useState3 = useState(null),
      phone = _useState3[0],
      setPhone = _useState3[1];

  var _useForm = useForm(),
      handleSubmit = _useForm.handleSubmit;

  var _useState4 = useState(false),
      isSubmit = _useState4[0],
      setSubmit = _useState4[1];

  var dispatch = useDispatch();

  var activeTheme = _get(setting, 'activeTheme');
  var enableFieldName = _get(setting, 'callThemeSettings.commonSettings.form_name_enable');
  var enableFieldEmail = _get(setting, 'callThemeSettings.commonSettings.form_email_enable');
  var enableFieldPhone = _get(setting, 'callThemeSettings.commonSettings.form_phone_enable');
  var requiredFieldName = _get(setting, 'callThemeSettings.commonSettings.form_name_require');
  var requiredFieldEmail = _get(setting, 'callThemeSettings.commonSettings.form_email_require');
  var requiredFieldPhone = _get(setting, 'callThemeSettings.commonSettings.form_phone_require');

  var onSubmit = function onSubmit() {
    var validateName = enableFieldName && requiredFieldName && _isEmpty(name) ? false : true;
    var validateEmail = enableFieldEmail && requiredFieldEmail && _isEmpty(email) ? false : true;
    var validatePhone = enableFieldPhone && requiredFieldPhone && _isEmpty(phone) ? false : true;
    setSubmit(true);
    if (!_includes([validateName, validatePhone, validateEmail], false)) {
      dispatch(leaveInformation({ name: name, email: email, phone: phone }, setting.channelId));
    }
  };

  var getInputTextColor = function getInputTextColor(setting) {
    var activeTheme = _get(setting, 'activeTheme');
    if (activeTheme === 'style-one') {
      return _get(setting, 'screenForm.inputTextColor');
    } else if (activeTheme === 'style-two') {
      return _get(setting, 'screenConversation.rightSideBackgroundColor');
    } else {
      return _get(setting, 'screenConversation.rightSideBackgroundColor');
    }
  };

  return React.createElement(
    'div',
    { className: styles.content },
    React.createElement(
      'div',
      { className: styles.userInfoFormContainer + ' bs-call-form' },
      React.createElement(
        'span',
        { className: styles.title },
        _get(setting, 'callThemeSettings.commonSettings.leave_info_tile')
      ),
      React.createElement(
        'div',
        { className: styles.form },
        React.createElement(
          'form',
          { onSubmit: handleSubmit(onSubmit), className: styles.userForm },
          React.createElement(
            'div',
            null,
            enableFieldName && React.createElement(FormInput, {
              style: activeTheme,
              backgroundColor: _get(setting, 'screenForm.inputBackgroundColor'),
              isRequired: requiredFieldName,
              textColor: getInputTextColor(setting),
              icon: React.createElement(SVG, { src: SVGName, style: { width: '12px', height: '12px' } }),
              placeholder: _get(setting, 'callThemeSettings.commonSettings.form_name_placeholder'),
              onChange: function onChange(e) {
                return setName(e.target.value);
              },
              isValid: isSubmit ? _isEmpty(name) : undefined,
              inputValue: name,
              placeholderColor: _get(setting, 'screenForm.placeholderTextColor')
            })
          ),
          React.createElement(
            'div',
            null,
            enableFieldEmail && React.createElement(FormInput, {
              style: activeTheme,
              backgroundColor: _get(setting, 'screenForm.inputBackgroundColor'),
              isRequired: requiredFieldEmail,
              textColor: getInputTextColor(setting),
              icon: React.createElement(SVG, { src: SVGEmail, style: { width: '16px', height: '11px' } }),
              placeholder: _get(setting, 'callThemeSettings.commonSettings.form_email_placeholder'),
              onChange: function onChange(e) {
                return setEmail(e.target.value);
              },
              isValid: isSubmit ? _isEmpty(email) : undefined,
              inputValue: email,
              placeholderColor: _get(setting, 'screenForm.placeholderTextColor')
            })
          ),
          React.createElement(
            'div',
            null,
            enableFieldPhone && React.createElement(FormInput, {
              style: activeTheme,
              backgroundColor: _get(setting, 'screenForm.inputBackgroundColor'),
              isRequired: requiredFieldPhone,
              textColor: getInputTextColor(setting),
              icon: React.createElement(SVG, { style: { width: '16.8px', height: '16.8px' }, src: SVGPhone }),
              placeholder: _get(setting, 'callThemeSettings.commonSettings.form_phone_placeholder'),
              onChange: function onChange(e) {
                return setPhone(e.target.value);
              },
              isValid: isSubmit ? _isEmpty(phone) : undefined,
              inputValue: phone,
              placeholderColor: _get(setting, 'screenForm.placeholderTextColor')
            })
          ),
          React.createElement(
            'div',
            null,
            React.createElement(Button, {
              width: '100%',
              setting: setting,
              sendIcon: true,
              borderRadius: 10,
              text: _get(setting, 'callThemeSettings.commonSettings.send_info_label')
            })
          )
        )
      ),
      React.createElement(
        'div',
        { className: styles.power },
        React.createElement(Power, { setting: setting })
      )
    )
  );
});