import _extends from 'babel-runtime/helpers/extends';
import _regeneratorRuntime from 'babel-runtime/regenerator';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import _find from 'lodash/find';
import _get from 'lodash/get';
import _includes from 'lodash/includes';
import React, { Component, useEffect } from 'react';
import Frame from 'react-frame-component';
import callIcon from '../icon/call-widget.svg';
import { checkMediaDevice, toggleCall, updateMediaPermission, startCall, stopCall, selectDevices } from '../../store/actions';
import { connect } from 'react-redux';

import { bindActionCreators } from 'redux';
import PageNoSupport from './components/no-support';
import PageGetPermission from './components/get-permission';
import PageStartCall from './components/start-call';
import PageInitCall from './components/init-call';
import PageCalling from './components/calling';
import PageComplete from './components/complete';
import PageBusy from './components/busy-line';
import { CALL_STATUS, CALL_STOP_REASON } from './constants';
import WidgetIcon from '../../commons/components/widget-icon';
import Layout from './components/layout';
import Tooltip from '../tooltip';

var PhoneCall = function (_Component) {
  _inherits(PhoneCall, _Component);

  function PhoneCall(props) {
    var _this2 = this;

    _classCallCheck(this, PhoneCall);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _this.handleClickCall = function () {
      _this.props.actions.toggleCall();
    };

    _this.handleGetPermission = function (_ref) {
      var isAllow = _ref.isAllow;

      _this.props.actions.updateMediaPermission(isAllow);
      _this.props.actions.checkMediaDevice();
    };

    _this.handleStartCall = function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(speakerId, microphoneId) {
        var setting;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                setting = _this.props.setting;

                _this.props.actions.startCall({ speakerId: speakerId, microphoneId: microphoneId }, setting.channelId);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this2);
      }));

      return function (_x, _x2) {
        return _ref2.apply(this, arguments);
      };
    }();

    _this.handleSelectDevices = function (speakerId, microphoneId) {
      _this.props.actions.selectDevices({ speakerId: speakerId, microphoneId: microphoneId });
    };

    _this.handleStopCall = function (stopReason, totalDuration) {
      if (_this.props.isInConfigurationMode) return;
      var _this$props = _this.props,
          callInfo = _this$props.callInfo,
          setting = _this$props.setting;

      _this.props.actions.stopCall(callInfo.callData, { stopReason: stopReason || CALL_STOP_REASON.SUBSCRIBER_STOP, totalDuration: totalDuration }, setting.channelId);
    };

    _this.calculateWidgetPosition = function () {
      var _this$props2 = _this.props,
          position = _this$props2.position,
          area = _this$props2.area,
          setting = _this$props2.setting,
          showChat = _this$props2.showChat,
          rect = _this$props2.rect,
          isSettingCollapse = _this$props2.isSettingCollapse,
          isSettingLeftBottom = _this$props2.isSettingLeftBottom,
          isSettingVertical = _this$props2.isSettingVertical,
          marginBottom = _this$props2.marginBottom,
          marginLeft = _this$props2.marginLeft,
          marginRight = _this$props2.marginRight;
      var iconSize = setting.iconSize,
          iconMargin = setting.iconMargin;

      var chatboxPosition = _get(setting, 'chatboxComponent.position');
      var leftBottomAlign = _get(setting, 'leftBottomAlign');
      var rightBottomAlign = _get(setting, 'rightBottomAlign');
      var windowWidth = rect.width;
      var bottom = 10;
      var right = 0;
      var width = 380;
      var isChatBoxInSameArea = area === chatboxPosition;
      var isLeftBottomAlignHorizontal = leftBottomAlign === 'horizontal';
      var isRightBottomAlignHorizontal = rightBottomAlign === 'horizontal';
      if (isSettingCollapse) {
        if (isSettingLeftBottom) {
          if (showChat) {
            if (isSettingVertical) {
              right = position.right - width - width - iconMargin - iconMargin;
              bottom = marginBottom;
            } else {
              bottom = marginBottom + 10 + iconSize;
              right = windowWidth - marginRight - width - 10 - width - 10;
            }
          } else {
            // Don't care the chatbox now in this logic branch
            if (isSettingVertical) {
              right = position.right - width - iconMargin;
              bottom = marginBottom;
            } else {
              bottom = marginBottom + 10 + iconSize;
              right = windowWidth - width - marginLeft;
            }
          }
        } else {
          if (showChat) {
            if (isSettingVertical) {
              right = position.right + 380 + marginRight + iconSize + 10;
              bottom = marginBottom;
            } else {
              bottom = marginBottom + 10 + iconSize;
              right = marginRight + width + 10;
            }
          } else {
            if (isSettingVertical) {
              bottom = marginBottom;
              right = marginRight + iconSize + 10;
            } else {
              bottom = marginBottom + iconSize + 10;
              right = marginRight;
            }
          }
        }
      } else {
        if (isSettingLeftBottom) {
          // If chatbox is in them same area and showing.
          if (showChat) {
            if (isSettingVertical) {
              bottom = marginBottom;
              right = position.right - width - width - iconMargin - iconMargin;
            } else {
              bottom = marginBottom + 10 + iconSize;
              right = windowWidth - marginLeft - 10 - width - width;
            }
          } else {
            // Don't care the chatbox now in this logic branch
            if (isSettingVertical) {
              bottom = marginBottom;
              right = windowWidth - width - 10 - marginLeft - iconSize;
            } else {
              bottom = marginBottom + iconSize + iconMargin;
              right = windowWidth - width - marginLeft;
            }
          }
        } else {
          if (showChat) {
            if (isSettingVertical) {
              bottom = marginBottom;
              right = marginBottom + 10 + +iconSize + width + 10;
            } else {
              right = width + 10 + marginRight;
              bottom = marginBottom + 10 + iconSize;
            }
          } else {
            if (isSettingVertical) {
              right = marginRight + 10 + iconSize;
              bottom = marginBottom;
            } else {
              right = marginRight;
              bottom = marginBottom + 10 + iconSize;
            }
          }
        }
      }
      return { bottom: bottom, right: right, width: width };
    };

    _this.state = { isHover: false };
    _this.iframeRef = React.createRef();
    return _this;
  }

  PhoneCall.prototype.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    if (prevProps.iframeStyles !== this.props.iframeStyles) {
      if (this.iframeRef.current) {
        var stylesheetNode = _find(this.iframeRef.current.node.contentDocument.styleSheets, function (node) {
          return _get(node, 'ownerNode.className') === 'iframe-boostcrm';
        });

        if (stylesheetNode && stylesheetNode.ownerNode) {
          stylesheetNode.ownerNode.outerHTML = this.props.iframeStyles;
        }
      }
    }
  };

  PhoneCall.prototype.componentDidMount = function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              this.props.actions.checkMediaDevice();

            case 1:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function componentDidMount() {
      return _ref3.apply(this, arguments);
    }

    return componentDidMount;
  }();

  PhoneCall.prototype.renderCallWidget = function renderCallWidget() {
    var _this3 = this,
        _ref4;

    var _props = this.props,
        demoData = _props.demoData,
        isInConfigurationMode = _props.isInConfigurationMode,
        isSettingLeftBottom = _props.isSettingLeftBottom;

    var showCall = _get(this.props, 'callInfo.isShowCall');
    var currentComponent = _get(demoData, 'currentComponent');
    var isShowCall = isInConfigurationMode && currentComponent === 'call' || showCall;
    var callInfo = isInConfigurationMode ? demoData.callInfo : this.props.callInfo;

    var isAllowAccessMicrophone = callInfo.isAllowAccessMicrophone,
        isDeviceSupport = callInfo.isDeviceSupport,
        devices = callInfo.devices,
        selectedMicrophoneId = callInfo.selectedMicrophoneId,
        selectedSpeakerId = callInfo.selectedSpeakerId;
    var _props2 = this.props,
        isMobileMode = _props2.isMobileMode,
        iframeStyles = _props2.iframeStyles,
        setting = _props2.setting;

    var _calculateWidgetPosit = this.calculateWidgetPosition(),
        bottom = _calculateWidgetPosit.bottom,
        right = _calculateWidgetPosit.right,
        width = _calculateWidgetPosit.width;

    var status = isInConfigurationMode ? demoData.status : _get(callInfo, 'callData.status');
    var isShowGetPermission = isInConfigurationMode ? status === CALL_STATUS.ACCESS_REQUEST : !isAllowAccessMicrophone;
    var isBorder = isShowGetPermission || _includes([CALL_STATUS.START_CALL, CALL_STATUS.CONNECTING], status);
    var isStartCall = !status || _includes([CALL_STATUS.START_CALL, CALL_STATUS.CONNECTING], status);
    var isLeftSite = _get(setting, 'display.position') === 'LEFT_BOTTOM';
    var backgroundColor = _includes([CALL_STATUS.INITIATED, CALL_STATUS.QUEUED, CALL_STATUS.RINGING, CALL_STATUS.IN_PROGRESS], status) ? _get(setting, 'iconBackgroundColor') || '#fff' : '#fff';

    var renderWhenDeviceSupports = function renderWhenDeviceSupports() {
      return React.createElement(
        Layout,
        {
          isShowCall: isShowCall,
          setting: setting,
          isMobileMode: isMobileMode,
          border: isBorder ? true : false,
          backgroundColor: backgroundColor,
          isLeftSite: isLeftSite
        },
        isShowGetPermission && React.createElement(PageGetPermission, { handleGetPermission: _this3.handleGetPermission.bind(_this3), setting: setting, isMobileMode: isMobileMode }),
        isStartCall && React.createElement(PageStartCall, {
          callInfo: callInfo,
          devices: devices,
          handleStartCall: _this3.handleStartCall.bind(_this3),
          handleSelectDevices: _this3.handleSelectDevices.bind(_this3),
          selectedMicrophoneId: selectedMicrophoneId,
          selectedSpeakerId: selectedSpeakerId,
          setting: setting,
          isMobileMode: isMobileMode
        }),
        _includes([CALL_STATUS.INITIATED, CALL_STATUS.QUEUED, CALL_STATUS.RINGING], status) && React.createElement(PageInitCall, { callInfo: callInfo, handleStopCall: _this3.handleStopCall.bind(_this3), setting: setting, isMobileMode: isMobileMode }),
        status === CALL_STATUS.IN_PROGRESS && React.createElement(PageCalling, { callInfo: callInfo, handleStopCall: _this3.handleStopCall.bind(_this3), setting: setting, isMobileMode: isMobileMode }),
        _includes([CALL_STATUS.BUSY, CALL_STATUS.ADMIN_CANCELED, CALL_STATUS.CUSTOMER_CANCELED, CALL_STATUS.NO_ANSWER, CALL_STATUS.BUSY_LEAVE_INFO, CALL_STATUS.FAILED], status) && React.createElement(PageBusy, { status: status, setting: setting, handleCallAgain: _this3.handleStartCall.bind(_this3), isMobileMode: isMobileMode }),
        _includes([CALL_STATUS.COMPLETED, CALL_STATUS.COMPLETED_RATING], status) && React.createElement(PageComplete, {
          status: status,
          setting: setting,
          callInfo: callInfo,
          handleMakeAnotherCall: _this3.handleStartCall.bind(_this3),
          isMobileMode: isMobileMode
        }),
        status === CALL_STATUS.NO_SUPPORT && React.createElement(PageNoSupport, { status: status, setting: setting, isMobileMode: isMobileMode })
      );
    };

    return React.createElement(
      Frame,
      {
        ref: this.iframeRef,
        initialContent: '<!DOCTYPE html><html><head>          \n          <link href="https://fonts.googleapis.com/css?family=Comfortaa&display=swap" rel="stylesheet">\n          <link href="https://fonts.googleapis.com/css?family=Roboto&display=swap" rel="stylesheet">\n          <link href="https://fonts.googleapis.com/css2?family=Open+Sans&display=swap" rel="stylesheet">\n          <style>\n            .bs-chatbox-input-container .ant-upload {\n              display: flex !important;\n            }\n            .bs-call-form .ant-form-item-control {\n              line-height: 31px !important;\n            }\n          </style>\n          ' + iframeStyles + '</head><body></body></html>\n        ',
        mountTarget: 'body',
        allowtransparency: 'yes',
        style: (_ref4 = {
          right: isMobileMode ? 0 : right + 'px',
          bottom: isMobileMode ? 0 : bottom + 'px',
          position: isMobileMode ? 'fixed' : 'absolute',
          width: isMobileMode ? '100%' : width + 'px',
          minHeight: '468px',
          maxHeight: isMobileMode ? '100%' : '468px',
          opacity: isShowCall ? '1' : '0',
          transform: isShowCall ? 'scale(1, 1)' : 'scale(0, 0)',
          height: isMobileMode ? '100%' : 'calc(100% - ' + bottom + 'px)',
          zIndex: 9999999,
          transition: 'opacity 0.8s',
          overflow: 'hidden',
          transformOrigin: isSettingLeftBottom ? 'bottom left' : 'bottom right'
        }, _ref4['transition'] = 'transform 0.3s ease-in-out,opacity 0.3s ease-in-out , box-shadow 0.3s ease-in-out', _ref4.maxWidth = 'unset', _ref4),
        frameBorder: '0'
      },
      React.createElement(
        'div',
        { style: { width: '100%', height: '100%' } },
        !isDeviceSupport && React.createElement(
          Layout,
          { setting: setting, isMobileMode: isMobileMode, backgroundColor: backgroundColor },
          React.createElement(PageNoSupport, { status: status, setting: setting, isMobileMode: isMobileMode })
        ),
        isDeviceSupport && renderWhenDeviceSupports()
      )
    );
  };

  PhoneCall.prototype.render = function render() {
    var _this4 = this;

    var _props3 = this.props,
        iframeStyles = _props3.iframeStyles,
        position = _props3.position,
        setting = _props3.setting,
        svgSize = _props3.svgSize;
    var iconBackgroundColor = setting.iconBackgroundColor,
        iconTextColor = setting.iconTextColor;

    var iconSize = _get(setting, 'iconSize');
    return React.createElement(
      'div',
      null,
      this.renderCallWidget(),
      React.createElement(
        Frame,
        {
          initialContent: '<!DOCTYPE html><html><head>' + iframeStyles + '</head><body></body></html>',
          mountTarget: 'body',
          style: {
            width: iconSize,
            height: iconSize,
            position: 'absolute',
            bottom: position.bottom + 'px',
            right: position.right + 'px',
            transition: 'opacity 0.8s',
            zIndex: '9999997',
            maxWidth: iconSize,
            minWidth: iconSize,
            minHeight: iconSize,
            maxHeight: iconSize
          },
          allowtransparency: 'yes',
          frameBorder: '0'
        },
        React.createElement(WidgetIcon, {
          iconBackgroundColor: iconBackgroundColor,
          iconSize: iconSize,
          iconSrc: callIcon,
          iconTextColor: iconTextColor,
          svgSize: svgSize,
          onClick: this.handleClickCall,
          handleHover: function handleHover(e) {
            return _this4.setState(e);
          }
        })
      ),
      this.state.isHover && React.createElement(Tooltip, _extends({}, this.props, this.state))
    );
  };

  return PhoneCall;
}(Component);

export default connect(function (store) {
  return {
    callInfo: _get(store, 'behavior.callInfo'),
    showChat: _get(store, 'behavior.showChat')
  };
}, function (dispatch) {
  return {
    actions: bindActionCreators({
      checkMediaDevice: checkMediaDevice,
      toggleCall: toggleCall,
      updateMediaPermission: updateMediaPermission,
      startCall: startCall,
      stopCall: stopCall,
      selectDevices: selectDevices
    }, dispatch)
  };
})(PhoneCall);