import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import _extends from 'babel-runtime/helpers/extends';
import _size from 'lodash/size';
import _filter from 'lodash/filter';
import _get from 'lodash/get';
import React, { Component, Suspense, lazy } from 'react';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import { DEMO_SCREEN_TYPES } from '../../commons/constants/demo-screen-types';

import ResizeObserver from 'react-resize-observer';
import { toggleTheSupportWidgetRequest } from '../../store/actions';
var ChatBox = lazy(function () {
  return import('../chat-box');
});
var PhoneCall = lazy(function () {
  return import('../phone-call');
});
var ScrollTop = lazy(function () {
  return import('../scroll-top');
});
var Messenger = lazy(function () {
  return import('../messenger');
});
var Support = lazy(function () {
  return import('../support');
});
var Zalo = lazy(function () {
  return import('../zalo');
});
var Instagram = lazy(function () {
  return import('../instagram');
});
var Email = lazy(function () {
  return import('../email');
});
var LinkedIn = lazy(function () {
  return import('../linkedIn');
});
var Skype = lazy(function () {
  return import('../skype');
});
var Sms = lazy(function () {
  return import('../sms');
});
var Twitter = lazy(function () {
  return import('../twitter');
});
var Viber = lazy(function () {
  return import('../viber');
});
var Youtube = lazy(function () {
  return import('../youtube');
});
var Tiktok = lazy(function () {
  return import('../tiktok');
});

// import "antd/dist/antd.css";
var POSITION_LEFT_BOTTOM = 'left-bottom';
var VERTICAL = 'vertical';
var HORIZONTAL = 'horizontal';
var MOBILE_WIDTH = 767;
var iconMargin = 10;
// const iconWidth = 60;
var boostcrmWidgets = ['chatbox', 'support', 'scrollTop', 'call'];
var socialWidgets = ['email', 'instagram', 'linkedin', 'skype', 'sms', 'twitter', 'viber', 'youtube', 'messenger', 'zalo', 'tiktok'];
function isInLeftBottom(component) {
  if (_get(component, 'position') === POSITION_LEFT_BOTTOM) {
    return true;
  }
  return false;
}

var getSvgSize = function getSvgSize(size) {
  var svgSize = 33;
  if (size === 50) {
    svgSize = svgSize - 5;
    return svgSize;
  }
  if (size === 70) {
    svgSize = svgSize + 5;
    return svgSize;
  }
  return svgSize;
};

var calculateMargin = function calculateMargin(rect, iconWidth, value, unit, isWidth) {
  switch (unit) {
    case 'em':
      return value * iconWidth || 10;
    case '%':
      return value * (isWidth ? rect.width : rect.height) / 100 || 10;
    case 'px':
    default:
      return value || 10;
  }
};

var display = {
  COLLAPSED: 'COLLAPSED',
  EXPANDED: 'EXPANDED'
};

var direction = {
  VERTICAL: 'VERTICAL',
  HORIZONTAL: 'HORIZONTAL'
};

var positionWidget = {
  RIGHT_BOTTOM: 'RIGHT_BOTTOM',
  LEFT_BOTTOM: 'LEFT_BOTTOM'
};

var supportComponent = {
  configurationData: {},
  enable: true,
  order: 99,
  position: 'right-bottom'
};

var getInitialStateFromProps = function getInitialStateFromProps(props) {
  var isFixedRect = props.isFixedRect,
      rect = props.rect,
      setting = props.setting,
      demoData = props.demoData,
      isInConfigurationMode = props.isInConfigurationMode,
      showSupportWidget = props.showSupportWidget;

  var _ref = demoData || {},
      currentComponent = _ref.currentComponent;

  var margins = setting.margins;

  var defaultDisplay = _get(setting, 'display.style');

  var bottom = margins.bottom,
      left = margins.left,
      right = margins.right;

  var iconWidth = _get(setting, 'iconSize');
  var marginRight = calculateMargin(rect, iconWidth, _get(right, 'value'), _get(right, 'cssUnit'), true);
  var marginLeft = calculateMargin(rect, iconWidth, _get(left, 'value'), _get(left, 'cssUnit'), true);
  var marginBottom = calculateMargin(rect, iconWidth, _get(bottom, 'value'), _get(bottom, 'cssUnit'), false);
  return _extends({
    setting: setting,
    demoData: demoData,
    isSkipHandlingCollapseExpandEvent: isInConfigurationMode && currentComponent !== DEMO_SCREEN_TYPES.GENERAL,
    isSettingCollapse: defaultDisplay === display.COLLAPSED,
    isCurrentExpand: isInConfigurationMode && currentComponent !== DEMO_SCREEN_TYPES.GENERAL || showSupportWidget,
    isSettingLeftBottom: _get(setting, 'display.position') === positionWidget.LEFT_BOTTOM,
    isSettingVertical: _get(setting, 'display.direction') === direction.VERTICAL,
    isEnableSocialWidgets: _get(setting, 'enableSocialWidgets'),
    marginRight: marginRight,
    marginLeft: marginLeft,
    marginBottom: marginBottom
  }, isFixedRect ? {
    rect: rect,
    isMobileMode: rect.width < MOBILE_WIDTH
  } : {});
};

var Widget = function (_Component) {
  _inherits(Widget, _Component);

  function Widget() {
    var _temp, _this, _ret;

    _classCallCheck(this, Widget);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _Component.call.apply(_Component, [this].concat(args))), _this), _this.state = {
      isMobileMode: false,
      rect: {
        width: 0,
        height: 0
      }
    }, _this.toggleCollapseExpandWidget = function () {
      if (_this.state.isSkipHandlingCollapseExpandEvent) {
        return;
      }

      if (_this.props.isInConfigurationMode) {
        _this.setState({
          isCurrentExpand: !_this.state.isCurrentExpand
        });
        return;
      }

      _this.props.actions.toggleTheSupportWidgetRequest(!_this.state.isCurrentExpand);
    }, _this.getWidgetComponent = function (name) {
      var _this$props$setting = _this.props.setting,
          chatboxComponent = _this$props$setting.chatboxComponent,
          messengerComponent = _this$props$setting.messengerComponent,
          zaloComponent = _this$props$setting.zaloComponent,
          callComponent = _this$props$setting.callComponent,
          scrollTopComponent = _this$props$setting.scrollTopComponent,
          tempWidgets = _this$props$setting.tempWidgets;

      var widgets = _filter(tempWidgets, function (widget) {
        return widget.type.indexOf(name) !== -1;
      });
      if (_size(widgets) > 0) {
        widgets = widgets.map(function (widget) {
          return { name: name, component: widget };
        });
      }
      switch (name) {
        case 'chatbox':
          return [{ name: name, component: chatboxComponent }];
        case 'call':
          return [{ name: name, component: callComponent }];
        case 'scrollTop':
          return [{ name: name, component: _extends({}, scrollTopComponent, { order: 0 }) }];
        case 'support':
          return [{ name: name, component: supportComponent }];
        case 'zalo':
          zaloComponent && widgets.push({ name: name, component: _extends({}, zaloComponent, { type: 'zalo_legacy' }) });
          return widgets;
        case 'messenger':
          messengerComponent && widgets.push({ name: name, component: _extends({}, messengerComponent, { type: 'messenger_legacy' }) });
          return widgets;
        case 'email':
        case 'instagram':
        case 'linkedin':
        case 'skype':
        case 'sms':
        case 'twitter':
        case 'youtube':
        case 'viber':
        case 'tiktok':
          return widgets;
      }
    }, _this.calculateAmountWidget = function (widgets) {
      widgets = widgets.filter(function (widget) {
        return widget.name !== 'support' && widget.name !== 'scrollTop' && _get(widget, 'component.enable') === true;
      });
      return widgets.length;
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  Widget.prototype.componentDidMount = function componentDidMount() {
    this.setState(getInitialStateFromProps(this.props));
  };

  Widget.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
    if (props.setting !== state.setting || props.demoData !== state.demoData) {
      return getInitialStateFromProps(props);
    }

    if (!props.isInConfigurationMode && props.showSupportWidget !== state.isCurrentExpand) {
      return {
        isCurrentExpand: props.showSupportWidget
      };
    }
    return null;
  };

  Widget.prototype.getComponentPosition = function getComponentPosition() {
    var _this2 = this;

    var _state = this.state,
        isSettingCollapse = _state.isSettingCollapse,
        isCurrentExpand = _state.isCurrentExpand,
        rect = _state.rect,
        isEnableSocialWidgets = _state.isEnableSocialWidgets,
        marginRight = _state.marginRight,
        marginLeft = _state.marginLeft,
        marginBottom = _state.marginBottom;


    var iconWidth = _get(this.props.setting, 'iconSize');
    var leftBottomList = [];
    var rightBottomList = [];
    var isLeftBottom = _get(this.props.setting, 'display.position') === positionWidget.LEFT_BOTTOM;
    var isVertical = _get(this.props.setting, 'display.direction') === direction.VERTICAL;

    var widgets = [].concat(boostcrmWidgets, socialWidgets);

    if (isSettingCollapse) {
      if (isLeftBottom) {
        widgets.forEach(function (name) {
          leftBottomList = [].concat(leftBottomList, _this2.getWidgetComponent(name));
        });
      } else {
        widgets.forEach(function (name) {
          rightBottomList = [].concat(rightBottomList, _this2.getWidgetComponent(name));
        });
      }
    } else {
      widgets.filter(function (name) {
        return name !== 'support';
      }).forEach(function (name) {
        if (isLeftBottom) {
          leftBottomList = [].concat(leftBottomList, _this2.getWidgetComponent(name));
        } else {
          rightBottomList = [].concat(rightBottomList, _this2.getWidgetComponent(name));
        }
      });
    }

    var amountWidget = this.calculateAmountWidget([].concat(leftBottomList, rightBottomList));

    if (amountWidget <= 1) {
      isSettingCollapse = false;
    }

    if (amountWidget > 6) {
      if (!isSettingCollapse) {
        if (isLeftBottom) {
          leftBottomList = [].concat(leftBottomList, [{ name: 'support', component: supportComponent }]);
        } else {
          rightBottomList = [].concat(rightBottomList, [{ name: 'support', component: supportComponent }]);
        }
      }
      isSettingCollapse = true;
    }

    if (isSettingCollapse) {
      leftBottomList = leftBottomList.filter(function (component) {
        return _get(component, 'component.enable') === true && (isCurrentExpand || _get(component, 'name') === 'support' || _get(component, 'name') === 'scrollTop');
      }).sort(function (a, b) {
        return _get(b, 'component.order') - _get(a, 'component.order');
      });

      rightBottomList = rightBottomList.filter(function (component) {
        return _get(component, 'component.enable') === true && (isCurrentExpand || _get(component, 'name') === 'support' || _get(component, 'name') === 'scrollTop');
      }).sort(function (a, b) {
        return _get(b, 'component.order') - _get(a, 'component.order');
      });
    } else {
      leftBottomList = leftBottomList.filter(function (component) {
        return _get(component, 'component.enable') === true;
      }).sort(function (a, b) {
        return _get(b, 'component.order') - _get(a, 'component.order');
      });

      rightBottomList = rightBottomList.filter(function (component) {
        return _get(component, 'component.enable') === true;
      }).sort(function (a, b) {
        return _get(b, 'component.order') - _get(a, 'component.order');
      });
    }
    var result = [];

    if (amountWidget <= 1) {
      leftBottomList = leftBottomList.filter(function (widget) {
        return widget.name !== 'support';
      });
      rightBottomList = rightBottomList.filter(function (widget) {
        return widget.name !== 'support';
      });
    }

    if (isSettingCollapse && amountWidget > 1) {
      leftBottomList.forEach(function (component, idx) {
        result.push({
          enable: true,
          position: isVertical || component.name === 'scrollTop' ? {
            right: rect.width - iconWidth - marginLeft,
            bottom: (iconWidth + iconMargin) * (!isVertical ? 1 : idx) + marginBottom
          } : {
            right: idx === 0 ? rect.width - (marginLeft + iconWidth) : rect.width - ((iconMargin + iconWidth) * idx + marginLeft + iconWidth),
            bottom: marginBottom
          },
          area: 'left-bottom',
          type: component.name,
          component: component.component
        });
      });
      rightBottomList.forEach(function (component, idx) {
        result.push({
          enable: true,
          position: isVertical || component.name === 'scrollTop' ? {
            right: marginRight,
            bottom: (iconWidth + iconMargin) * (!isVertical ? 1 : idx) + marginBottom
          } : {
            right: (iconWidth + iconMargin) * idx + marginRight,
            bottom: marginBottom
          },
          area: 'right-bottom',
          type: component.name,
          component: component.component
        });
      });
    } else {
      leftBottomList.forEach(function (component, idx) {
        result.push({
          enable: true,
          position: isVertical ? {
            right: rect.width - iconWidth - marginLeft,
            bottom: (iconWidth + iconMargin) * idx + marginBottom
          } : {
            right: idx === 0 ? rect.width - (marginLeft + iconWidth) : rect.width - ((iconMargin + iconWidth) * idx + marginLeft + iconWidth),
            bottom: marginBottom
          },
          area: 'left-bottom',
          type: component.name,
          component: component.component
        });
      });

      rightBottomList.forEach(function (component, idx) {
        result.push({
          enable: true,
          position: isVertical ? {
            right: marginRight,
            bottom: (iconWidth + iconMargin) * idx + marginBottom
          } : {
            right: (iconWidth + iconMargin) * idx + marginRight,
            bottom: marginBottom
          },
          area: 'right-bottom',
          type: component.name,
          component: component.component
        });
      });
    }

    return result;
  };

  Widget.prototype.render = function render() {
    var _this3 = this;

    var widgets = this.getComponentPosition();
    var isFixedRect = this.props.isFixedRect;

    var iconSize = _get(this.props.setting, 'iconSize');
    var svgSize = getSvgSize(iconSize);
    return React.createElement(
      'div',
      { style: { width: '100%', height: '100%', position: 'relative' } },
      !isFixedRect && React.createElement(ResizeObserver, {
        onPosition: function onPosition(rect) {
          _this3.setState({
            rect: {
              height: rect.top,
              width: rect.left
            },
            isMobileMode: rect.left <= MOBILE_WIDTH
          });
        }
      }),
      (widgets || []).map(function (widget, index) {
        switch (widget.type) {
          case 'scrollTop':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(ScrollTop, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'call':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(PhoneCall, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'chatbox':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(ChatBox, _extends({ index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'zalo':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Zalo, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'messenger':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Messenger, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'email':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Email, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'linkedin':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(LinkedIn, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'skype':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Skype, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'sms':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Sms, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'twitter':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Twitter, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'viber':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Viber, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'youtube':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Youtube, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'instagram':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Instagram, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'tiktok':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Tiktok, _extends({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'support':
            return React.createElement(
              Suspense,
              { key: index, index: index, fallback: React.createElement('div', null) },
              React.createElement(Support, _extends({
                key: index
              }, _this3.props, _this3.state, widget, {
                svgSize: svgSize,
                isCurrentExpand: _this3.state.isCurrentExpand,
                toggleCollapseExpandWidget: _this3.toggleCollapseExpandWidget
              }))
            );
        }
      })
    );
  };

  return Widget;
}(Component);

export default connect(function (store) {
  return {
    showChat: _get(store, 'behavior.showChat'),
    showSupportWidget: _get(store, 'behavior.showSupportWidget'),
    numberOfNewMessages: _get(store, 'messages.numberOfNewMessages')
  };
}, function (dispatch) {
  return {
    actions: bindActionCreators({
      toggleTheSupportWidgetRequest: toggleTheSupportWidgetRequest
    }, dispatch)
  };
})(Widget);