import _extends from 'babel-runtime/helpers/extends';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import _typeof from 'babel-runtime/helpers/typeof';
import _regeneratorRuntime from 'babel-runtime/regenerator';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _cloneDeep from 'lodash/cloneDeep';
import _find from 'lodash/find';
import _get from 'lodash/get';

var _this = this;

import React, { Suspense, lazy } from 'react';
import { Provider } from 'react-redux';
import { createStore, combineReducers, applyMiddleware } from 'redux';
import thunk from 'redux-thunk';

import behavior from './store/reducers/behavior.reducer';
import messages from './store/reducers/message.reducer';
import { composeWithDevTools } from 'redux-devtools-extension';
import { initWSClient, closeWSClient, setMessageHanlder, sendWebsiteEvent, disconnectWSClient, reconnectWSClient } from '../src/connections/websocket';
import WidgetContext from './widget-context';
import { setLSVariable, getLSVariable } from './commons/utils/local-storage';
import { setChannelId, setTenantId, setConversationId } from './commons/tenant-info';
import { LOCAL_STORAGE_VARIABLES, getLocalStorageVariables } from './commons/constants/local-storage';
import { WEBSITE_EVENTS } from './commons/constants/website-events';
import { GROUP, TYPE } from './commons/constants/websocket';
import { STARTING_POINT_TYPES } from './commons/constants/starting-point-types';
import { sendAckReceivingMessage } from './store/actions/ws.actions';
import clearEndSlash from './commons/utils/clear-end-slash';
import runFunAfter from './commons/utils/run-fun-after';
import { NetworkDetector } from './commons/utils/network-detector';
import { triggerTheStartingPoint, triggerTheWelcomeMessageWhenVisitPage } from './store/actions/start-conversation.actions';
var Widget = lazy(function () {
  return import('./components/widget');
});
var Conversation = lazy(function () {
  return import('./components/conversation');
});

var getHrefWithoutQueryAndParam = function getHrefWithoutQueryAndParam() {
  return '' + window.location.origin + window.location.pathname;
};

var getListStartingPoints = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(host, channelId, templateId) {
    var results, pageIndex, pageSize, totalReturnedItems, response, returnedItems;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            results = [];
            pageIndex = 1;
            pageSize = 10;
            totalReturnedItems = 0;

          case 4:
            _context.next = 6;
            return fetch(host + '/channel/website/starting-points?channelId=' + channelId + '&templateId=' + templateId + '&pageIndex=' + pageIndex, {
              method: 'GET'
            });

          case 6:
            response = _context.sent;

            ++pageIndex;
            _context.next = 10;
            return response.json();

          case 10:
            returnedItems = _context.sent;

            Array.from(returnedItems || []).forEach(function (item) {
              if (item.active) {
                results.push(item);
              }
            });
            totalReturnedItems = returnedItems.length;

          case 13:
            if (totalReturnedItems === pageSize) {
              _context.next = 4;
              break;
            }

          case 14:
            return _context.abrupt('return', results);

          case 15:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, _this);
  }));

  return function getListStartingPoints(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

function findValidStartingPoint(startingPoints, type, opts) {
  for (var _iterator = startingPoints, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref2;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref2 = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref2 = _i.value;
    }

    var startPoint = _ref2;

    if (type === STARTING_POINT_TYPES.VISITOR_CLICK_ELEMENT) {
      var id = opts.id,
          className = opts.className;

      if (id && id === _get(startPoint, 'detail.id')) return startPoint;
      if (className && className === _get(startPoint, 'detail.className')) return startPoint;
    }

    if (type === STARTING_POINT_TYPES.VISITOR_VISIT_URLS) {
      var _ret = function () {
        var url = opts.url;
        // console.log("url:", url);

        var requiredUrls = _get(startPoint, 'detail.urls') || [];
        var lowercaseUrlWithoutEndSlash = toLower(clearEndSlash(url));
        // console.log("lowercaseUrlWithoutEndSlash:", lowercaseUrlWithoutEndSlash);
        if (_find(requiredUrls, function (defineUrl) {
          return toLower(clearEndSlash(defineUrl)) === lowercaseUrlWithoutEndSlash;
        })) {
          // console.log("found starting point");
          return {
            v: startPoint
          };
        }
      }();

      if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
    }
  }
  return null;
}

var ConnectedWidget = function (_React$Component) {
  _inherits(ConnectedWidget, _React$Component);

  function ConnectedWidget(props) {
    _classCallCheck(this, ConnectedWidget);

    var _this2 = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _this2.networkChangesHandler = function (isOnline) {
      if (!isOnline) {
        disconnectWSClient();
      } else {
        reconnectWSClient();
      }
    };

    var _this2$props = _this2.props,
        isInConfigurationMode = _this2$props.isInConfigurationMode,
        isSimulator = _this2$props.isSimulator,
        onInitConversationSuccess = _this2$props.onInitConversationSuccess,
        setting = _this2$props.setting;
    var host = setting.host,
        tenantId = setting.tenantId,
        channelId = setting.channelId,
        automation = setting.automation;

    var isDisableLiveChat = !_get(setting, 'chatboxComponent.enable');
    var isDisableCall = !_get(setting, 'callComponent.enable');

    _this2.state = {
      online: true,
      isReconnecting: false,
      isConnectFailed: false,
      iframeStylesInnerHtml: '',
      iframeStyles: ''
    };

    var reducer = combineReducers({ behavior: behavior, messages: messages });
    if (isInConfigurationMode) {
      _this2.store = createStore(function () {}, process.env.NODE_ENV !== 'production' ? composeWithDevTools(applyMiddleware(thunk)) : applyMiddleware(thunk));
      return _possibleConstructorReturn(_this2);
    }

    var styleTag = props.pageDocument.head.querySelector('.boostcrm-stylesheet');
    if (styleTag) {
      var observer = new MutationObserver(function () {
        _this2.setState({
          iframeStylesInnerHtml: _this2.state.iframeStylesInnerHtml + styleTag.innerHTML,
          iframeStyles: '<style type="text/css" class="iframe-boostcrm">\n            .frame-content {\n              height: 100%;\n            }  \n            body {\n              font-family: \'Roboto !important\';\n            }\n            body {\n              background: none !important;\n              background-color: transparent !important;    \n              overflow:hidden\n            }\n            ' + (_this2.state.iframeStylesInnerHtml + styleTag.innerHTML) + '\n          </style>'
        });
        styleTag.innerHTML = '';
      });
      observer.observe(styleTag, { characterData: false, childList: true, attributes: false });
    }

    setTenantId(tenantId);
    setChannelId(channelId);
    setLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.IS_SIMULATOR, channelId), isSimulator);
    var persistentState = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.STORE_STATE, channelId));
    setLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CHANNEL_ID, channelId), channelId);
    setLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.TENANT_ID, channelId), tenantId);
    setLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.HOST, channelId), host);

    if (isSimulator) {
      persistentState = {};
    }
    if (_get(persistentState, ['behavior', 'callInfo', 'callData'])) {
      persistentState.behavior.callInfo.callData = null;
    }
    _this2.store = createStore(reducer, persistentState || {}, process.env.NODE_ENV !== 'production' ? composeWithDevTools(applyMiddleware(thunk)) : applyMiddleware(thunk));

    if (_this2.store) {
      _this2.store.subscribe(function () {
        var data = _cloneDeep(_this2.store.getState());
        setLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.STORE_STATE, channelId), data);
      });
    }

    if (!isSimulator && isDisableCall && isDisableLiveChat) return _possibleConstructorReturn(_this2);

    initWSClient(host, _this2.props.setting.channelToken, channelId, isSimulator, null, function (newConversationId) {
      // console.log("Start new conversation: ", newConversationId);
      setConversationId(newConversationId);
      newConversationId && onInitConversationSuccess && onInitConversationSuccess(newConversationId);
    }, function () {
      console.log('set online false 1');
      _this2.setState({
        online: false,
        isConnectFailed: true,
        isReconnecting: false
      });
    }, function () {
      console.log('set online false 2');
      _this2.setState({
        online: false,
        isConnectFailed: false,
        isReconnecting: true
      });
    }, function () {
      console.log('set online true');
      _this2.setState({
        online: true,
        isConnectFailed: false,
        isReconnecting: false
      });
    });

    setMessageHanlder(function (message) {
      var actionType = 'WS_' + _get(message, 'group') + '_' + _get(message, 'type');
      _this2.store.dispatch({
        type: actionType,
        payload: message.payload
      });

      if (_get(message, 'group') === GROUP.MESSAGE && _get(message, 'type') === TYPE.MESSAGE.NEW_MESSAGE_FROM_ADMIN) {
        _this2.store.dispatch(sendAckReceivingMessage(message.payload, channelId));
      }
    });

    if (!isSimulator) {
      sendWebsiteEvent(WEBSITE_EVENTS.VISIT_URL, {
        url: getHrefWithoutQueryAndParam(),
        loadDate: new Date(),
        tenantInfo: { channelId: channelId }
      });

      runFunAfter(function () {
        _this2.store.dispatch && _this2.store.dispatch(triggerTheWelcomeMessageWhenVisitPage(setting));
      }, 1000);

      // Get list of starting points here.

      var _ref3 = automation || {},
          enabled = _ref3.enabled,
          templateId = _ref3.templateId;

      if (enabled && templateId) {
        getListStartingPoints(host, setting.channelId, templateId).then(function (startingPoints) {
          if (startingPoints.length === 0) return;

          // Check if needed to trigger event 'visit-urls'
          var startingPointForVisitURLs = findValidStartingPoint(startingPoints, STARTING_POINT_TYPES.VISITOR_VISIT_URLS, {
            url: getHrefWithoutQueryAndParam()
          });

          startingPointForVisitURLs && runFunAfter(function () {
            _this2.store.dispatch && _this2.store.dispatch(triggerTheStartingPoint(STARTING_POINT_TYPES.VISITOR_VISIT_URLS, startingPointForVisitURLs.id, setting));
          }, 1000);

          var pageDocument = props.pageDocument;

          if (pageDocument) {
            pageDocument.addEventListener('click', function (e) {
              if (!e) return;
              var toElement = e.toElement;
              if (!toElement) return;
              var id = toElement.id;
              var className = toElement.className;
              var validStartPoint = findValidStartingPoint(startingPoints, STARTING_POINT_TYPES.VISITOR_CLICK_ELEMENT, {
                id: id,
                className: className
              });
              validStartPoint && _this2.store.dispatch && _this2.store.dispatch(triggerTheStartingPoint(STARTING_POINT_TYPES.VISITOR_CLICK_ELEMENT, validStartPoint.id, setting));
            }, true);
          }
        });
      }
    }
    return _this2;
  }

  ConnectedWidget.prototype.componentWillUnmount = function componentWillUnmount() {
    closeWSClient();
  };

  ConnectedWidget.prototype.render = function render() {
    var _props = this.props,
        setting = _props.setting,
        isSimulator = _props.isSimulator,
        isInConfigurationMode = _props.isInConfigurationMode;
    var _state = this.state,
        online = _state.online,
        isReconnecting = _state.isReconnecting,
        isConnectFailed = _state.isConnectFailed,
        iframeStyles = _state.iframeStyles;

    return React.createElement(
      Provider,
      { store: this.store },
      React.createElement(
        WidgetContext.Provider,
        {
          value: {
            setting: setting,
            isSimulator: isSimulator,
            online: online,
            isReconnecting: isReconnecting,
            isConnectFailed: isConnectFailed,
            isInConfigurationMode: isInConfigurationMode
          }
        },
        isSimulator && React.createElement(
          Suspense,
          { fallback: React.createElement('div', null) },
          React.createElement(Conversation, _extends({
            online: online,
            isReconnecting: isReconnecting,
            isConnectFailed: isConnectFailed,
            isSimulator: true,
            isMobileMode: true,
            iframeStyles: iframeStyles,
            rect: { width: 500, height: 600 }
          }, this.props))
        ),
        !isSimulator && React.createElement(
          Suspense,
          { fallback: React.createElement('div', null) },
          React.createElement(Widget, _extends({}, this.props, { iframeStyles: iframeStyles, rootWindow: window, rootDocument: document }))
        ),
        React.createElement(NetworkDetector, { onChange: this.networkChangesHandler })
      )
    );
  };

  return ConnectedWidget;
}(React.Component);

export { ConnectedWidget as default };