import _regeneratorRuntime from 'babel-runtime/regenerator';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _get from 'lodash/get';
import * as actions from './action-type';
import { createAction } from 'redux-actions';

import { sendMessage } from '../../connections/websocket';
import { GROUP, TYPE } from '../../commons/constants/websocket';
import { getTenantInfo } from '../../commons/tenant-info';
import { updateHasTriggeredWelcomeMessage } from './start-conversation.actions';
import { isDisablePreChatForm, isLastTimeMoreThanAPeriod, isTriggerWelcomeBackMessageWhenClickIcon, isTriggerWelcomeMessageWhenClickIcon } from '../../commons/utils';
import getTimeDuration from '../../commons/utils/get-time-duration';

export var onScrollToBottom = createAction(actions.ON_SCROLL_TO_BOTTOM);
export var toggleChatRequest = createAction(actions.TOGGLE_CHAT + '.REQUEST');

export var toggleTheSupportWidgetRequest = createAction(actions.TOGGLE_SUPPORT_WIDGET + '.REQUEST');

export function toggleChat(setting) {
  return function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch, getState) {
      var showChat, state, hasTriggeredWelcomeMessage, alreadyStartConversation, delay;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              showChat = _get(state, 'behavior.showChat');

              dispatch(toggleChatRequest(!showChat));

              if (setting.isEnableAutomation) {
                _context.next = 5;
                break;
              }

              console.log('Ingore because not enable bot now');
              return _context.abrupt('return');

            case 5:
              if (!showChat) {
                _context.next = 7;
                break;
              }

              return _context.abrupt('return');

            case 7:
              state = getState();
              hasTriggeredWelcomeMessage = _get(state, 'behavior.hasTriggeredWelcomeMessage');
              alreadyStartConversation = _get(state, 'behavior.alreadyStartConversation');

              if (!(!isDisablePreChatForm(setting) && !alreadyStartConversation)) {
                _context.next = 13;
                break;
              }

              console.log('Ingore because the pre-chat form is enable & the conversation has not started');
              return _context.abrupt('return');

            case 13:
              if (!(!isTriggerWelcomeMessageWhenClickIcon(setting) && !isTriggerWelcomeBackMessageWhenClickIcon(setting))) {
                _context.next = 16;
                break;
              }

              console.log('Ingore because dont enable welcome message & welcome back message when revisit');
              return _context.abrupt('return');

            case 16:
              if (!(isTriggerWelcomeMessageWhenClickIcon(setting) && !hasTriggeredWelcomeMessage)) {
                _context.next = 20;
                break;
              }

              sendMessage({
                group: GROUP.MESSAGE,
                type: TYPE.MESSAGE.TRIGGER_WELCOME_MESSAGE,
                payload: {},
                tenantInfo: getTenantInfo()
              });
              dispatch(updateHasTriggeredWelcomeMessage());
              return _context.abrupt('return');

            case 20:
              if (!(hasTriggeredWelcomeMessage && isTriggerWelcomeBackMessageWhenClickIcon(setting))) {
                _context.next = 25;
                break;
              }

              delay = getTimeDuration(_get(setting, 'screenWelcomeBackMessages.trigger.time'), _get(setting, 'screenWelcomeBackMessages.trigger.unitOfTime'));

              console.log('...Trigger the welcome back message when click if:', delay);
              if (isLastTimeMoreThanAPeriod(_get(state, 'behavior.lastTimeTriggeredWelcomeMessage'), delay)) {
                console.log('Trigger the welcome back message');
                sendMessage({
                  group: GROUP.MESSAGE,
                  type: TYPE.MESSAGE.TRIGGER_WELCOME_BACK_MESSAGE,
                  payload: {},
                  tenantInfo: getTenantInfo()
                });
                dispatch(updateHasTriggeredWelcomeMessage());
              }
              return _context.abrupt('return');

            case 25:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }();
}

export function toggleCall() {
  return {
    type: actions.TOGGLE_CALL
  };
}

export function settingSound() {
  return {
    type: actions.SOUND_SETTING
  };
}

export var updateMediaPermission = createAction(actions.UPDATE_MEDIA_PERMISSION + '.REQUEST');

export var selectDevices = createAction(actions.SELECT_DEVICE + '.REQUEST');