import _extends from 'babel-runtime/helpers/extends';
import _regeneratorRuntime from 'babel-runtime/regenerator';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _join from 'lodash/join';
import _size from 'lodash/size';
import _get from 'lodash/get';
import _includes from 'lodash/includes';

var _this = this;

import * as actions from './action-type';
import { createAction } from 'redux-actions';
import { sendMessage } from '../../connections/websocket';
import { GROUP, TYPE } from '../../commons/constants/websocket';
import { MESSAGE_TYPES } from '../../commons/constants/message-type';
import { getTenantInfo } from '../../commons/tenant-info';
import * as FormData from 'form-data';
import { getLSVariable } from '../../commons/utils/local-storage';
import { LOCAL_STORAGE_VARIABLES, getLocalStorageVariables } from '../../commons/constants/local-storage';

import { MESSAGE_STATUS } from '../../commons/constants/message-status';

export var sendTextMessageRequest = createAction(actions.SEND_TEXT_MESSAGE + '.REQUEST');
export var sendTextMessageSuccess = createAction(actions.SEND_TEXT_MESSAGE + '.SUCCESS');
export var sendTextMessage = function sendTextMessage(text) {
  return function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch, store) {
      var reqPayload;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              // console.log("sendTextMessage:", text, getTenantInfo());
              reqPayload = { type: MESSAGE_TYPES.TEXT, content: { text: text }, externalId: new Date().getTime() };
              _context.next = 3;
              return dispatch(sendTextMessageRequest(reqPayload));

            case 3:
              sendMessage({
                group: GROUP.MESSAGE,
                type: TYPE.MESSAGE.NEW_MESSAGE_FROM_CHATBOX,
                payload: reqPayload,
                tenantInfo: getTenantInfo()
              });

            case 4:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }();
};

export var sendAttachmentRequest = createAction(actions.SEND_ATTACHMENT + '.REQUEST');

export var sendAttachmentSuccess = createAction(actions.SEND_ATTACHMENT + '.SUCCESS');

export var sendAttachmentError = createAction(actions.SEND_ATTACHMENT + '.ERROR');

export var sendAttachment = function sendAttachment(text, files, channelKey) {
  return function (dispatch, store) {
    // console.log("sendAttachment.text: ", text);
    // console.log("sendAttachment.files: ", files);
    var isSimulator = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.IS_SIMULATOR, channelKey));
    // Get tenantId ,conversationId, channelId
    var channelId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CHANNEL_ID, channelKey));
    var tenantId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.TENANT_ID, channelKey));
    var conversationId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey));
    // console.log("sendAttachment", channelId, tenantId, conversationId);
    var form = new FormData();
    form.append('tenantId', tenantId);
    form.append('channelId', channelId);
    form.append('conversationId', conversationId);

    var listUUIDOrder = [];
    files.map(function (file) {
      form.append('files', file.originalFile);

      var reqPayload = void 0;
      var isImage = _get(file, 'isImage');
      var externalId = new Date().getTime();
      // console.log(`externalId`, externalId);

      isImage ? reqPayload = {
        type: MESSAGE_TYPES.IMAGE,
        content: {
          url: null
        },
        externalId: externalId
      } : reqPayload = {
        type: MESSAGE_TYPES.FILE,
        content: {
          name: file.name,
          url: null,
          type: _get(file, ['originalFile', 'type']),
          size: _get(file, ['originalFile', 'size'])
        },
        externalId: externalId
      };

      listUUIDOrder = isEmpty(listUUIDOrder) ? [externalId] : [].concat(listUUIDOrder, [externalId]);
      dispatch(sendAttachmentRequest(reqPayload));
    });

    form.append('listUUIDOrder', _join(listUUIDOrder, ','));
    if (!isEmpty(text)) {
      form.append('type', MESSAGE_TYPES.TEXT);
      form.append('content', text);

      var reqPayload = { type: MESSAGE_TYPES.TEXT, content: { text: text } };
      dispatch(sendAttachmentRequest(reqPayload));
    }
    // Submit form data
    var host = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.HOST, channelKey));
    var request = new XMLHttpRequest();
    request.open('POST', host + '/conversations/messages-from-chatbox');

    request.onreadystatechange = function () {
      if (request.readyState === 4) {
        var res = JSON.parse(_get(request, 'response'));
        var error = _get(res, 'error');
        if (error) {
          dispatch(sendAttachmentError({ res: res }));
          return;
        }
        dispatch(sendAttachmentSuccess({ res: res }));
      }
    };
    request.send(form);
  };
};

export var sendButtonMessage = function sendButtonMessage(buttonData) {
  return function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(dispatch, store) {
      var externalId, reqPayload;
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              externalId = new Date().getTime();
              reqPayload = {
                type: MESSAGE_TYPES.TEXT,
                content: { text: _get(buttonData, 'label') },
                externalId: externalId
              };
              _context2.next = 4;
              return dispatch(sendTextMessageRequest(reqPayload));

            case 4:
              sendMessage({
                group: GROUP.MESSAGE,
                type: TYPE.MESSAGE.NEW_MESSAGE_FROM_CHATBOX,
                payload: {
                  type: 'button',
                  content: buttonData,
                  externalId: externalId
                },
                tenantInfo: getTenantInfo()
              });

            case 5:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function (_x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }();
};

var _sendMessage = function _sendMessage(payload) {
  sendMessage({
    group: GROUP.MESSAGE,
    type: TYPE.MESSAGE.UPDATE_MESSAGE_STATUS,
    payload: payload,
    tenantInfo: getTenantInfo()
  });
};

var _updateLastMessageStatus = function _updateLastMessageStatus(_ref3) {
  var lastMessage = _ref3.lastMessage,
      getState = _ref3.getState,
      dispatch = _ref3.dispatch;

  if (!lastMessage) return;
  var lastMessageReceiveStatus = _get(lastMessage, 'status');
  var senderType = _get(lastMessage, 'senderType');
  var showChat = _get(getState(), 'behavior.showChat');
  if (senderType !== 1 && !_includes([MESSAGE_STATUS.READ, MESSAGE_STATUS.FAILED], lastMessageReceiveStatus) && showChat && !document.hidden) {
    _sendMessage(_extends({}, lastMessage, { status: MESSAGE_STATUS.READ }));
    dispatch(resetNumberOfNewMessages());
  }
};

export var getMessagesRequest = createAction('GET_MESSAGES_REQUEST');
export var getMessagesSuccess = createAction('GET_MESSAGES_SUCCESS');
export var getMessagesError = createAction('GET_MESSAGES_ERROR');
export var getMessages = function getMessages(isLoadingMessageHistory, channelKey) {
  return function () {
    var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(dispatch, getState) {
      var isSimulator, persistentState, conversationId, tenantId, channelId, host, messages, reqPayload, request;
      return _regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              isSimulator = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.IS_SIMULATOR, channelKey));
              persistentState = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.STORE_STATE, channelKey));
              conversationId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey));
              tenantId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.TENANT_ID, channelKey));
              channelId = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.CHANNEL_ID, channelKey));
              host = getLSVariable(getLocalStorageVariables(LOCAL_STORAGE_VARIABLES.HOST, channelKey));
              messages = _get(persistentState, ['messages', 'messages']);
              reqPayload = {
                conversationId: conversationId,
                tenantId: tenantId,
                channelId: channelId,
                isLoadingMessageHistory: isLoadingMessageHistory
              };

              dispatch(getMessagesRequest(reqPayload));

              request = new XMLHttpRequest();

              request.open('GET', host + '/messages/widget_messages?conversationId=' + conversationId + '&channelId=' + channelId + '&tenantId=' + tenantId + '&pageSize=10' + (isLoadingMessageHistory ? '&skip=' + _size(messages) : ''));

              request.onreadystatechange = function () {
                if (request.readyState === 4) {
                  var res = JSON.parse(_get(request, 'response'));
                  var error = _get(res, 'error');
                  if (error) {
                    dispatch(getMessagesError({ res: res }));
                    return;
                  }
                  dispatch(getMessagesSuccess(res, reqPayload));
                  var lastMessage = res[0];
                  !isLoadingMessageHistory && lastMessage && _updateLastMessageStatus({ lastMessage: lastMessage, getState: getState, dispatch: dispatch });
                }
              };
              request.send();

            case 13:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, _this);
    }));

    return function (_x5, _x6) {
      return _ref4.apply(this, arguments);
    };
  }();
};

export var resetNumberOfNewMessages = createAction('RESET_NUMBER_OF_NEW_MESSSAGES');