import _regeneratorRuntime from 'babel-runtime/regenerator';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _find from 'lodash/find';
import _isEmpty from 'lodash/isEmpty';
import _get from 'lodash/get';
import * as actions from './action-type';
import { createAction } from 'redux-actions';
import { sendMessage, triggerStartingPoint } from '../../connections/websocket';
import { BUTTON_STARTS, GROUP, TYPE } from '../../commons/constants/websocket';
import { getTenantInfo } from '../../commons/tenant-info';

import { saveUserInfo } from './save-user-info.actions';
import runFunAfter from '../../commons/utils/run-fun-after';
import getTimeDuration from '../../commons/utils/get-time-duration';
import { isDisablePreChatForm, isLastTimeMoreThanOneDay, isTriggerWelcomeMessageWhenVisitPage, isTriggerWelcomeBackMessageWhenVisitPage, isLastTimeMoreThanAPeriod } from '../../commons/utils';
import { STARTING_POINT_TYPES } from '../../commons/constants/starting-point-types';

import { sendTextMessage } from './message.actions';

export var startConversationRequest = createAction(actions.START_CONVERSATION + '.REQUEST');

export var updateHasTriggeredWelcomeMessage = createAction(actions.UPDATE_HAS_TRIGGERED_WELCOME_MESSAGE + '.SUCCESS');

export var trackingTheTriggerEvent = createAction(actions.TRACK_THE_TRIGGER_EVENT + '.SUCCESS');

export var startConversation = function startConversation(fromButton, _ref) {
  var name = _ref.name,
      email = _ref.email,
      phone = _ref.phone,
      isWorkingDay = _ref.isWorkingDay,
      leaveMessage = _ref.leaveMessage;

  return function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch) {
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return dispatch(startConversationRequest({ name: name, email: email, phone: phone, fromButton: fromButton, isWorkingDay: isWorkingDay }));

            case 2:
              if (!(!_isEmpty(name) || !_isEmpty(email) || !_isEmpty(phone))) {
                _context.next = 5;
                break;
              }

              _context.next = 5;
              return dispatch(saveUserInfo(name, email, phone));

            case 5:

              fromButton === BUTTON_STARTS.START_CONVERSATION && sendMessage({
                group: GROUP.MESSAGE,
                type: TYPE.MESSAGE.START_CONVERSATION,
                payload: { displayName: name, email: email, phone: phone },
                tenantInfo: getTenantInfo()
              });

              // Check whether it's needed to trigger the welcome message
              if (fromButton === BUTTON_STARTS.CHAT_WITH_BOT || fromButton === BUTTON_STARTS.START_CONVERSATION && isWorkingDay) {
                console.log('Trigger the welcome message');
                runFunAfter(function () {
                  sendMessage({
                    group: GROUP.MESSAGE,
                    type: TYPE.MESSAGE.TRIGGER_WELCOME_MESSAGE,
                    payload: {},
                    tenantInfo: getTenantInfo()
                  });
                  dispatch(updateHasTriggeredWelcomeMessage());
                }, 200);
              }

              if (fromButton === BUTTON_STARTS.LEAVE_MESSAGE) {
                console.log('BUTTON_STARTS.LEAVE_MESSAGE: ', leaveMessage);
                runFunAfter(function () {
                  dispatch(sendTextMessage(leaveMessage));
                }, 200);
              }

            case 8:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x) {
      return _ref2.apply(this, arguments);
    };
  }();
};

export var triggerTheStartingPoint = function triggerTheStartingPoint(startingPointType, startingPointId, setting) {
  return function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(dispatch, getState) {
      var state, alreadyStartConversation, trackingTriggerEvents, _ref4, lastTime;

      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              console.log('triggerTheStartingPoint:', startingPointId);

              if (setting.isEnableAutomation) {
                _context2.next = 4;
                break;
              }

              console.log('Ingore because not enable bot now');
              return _context2.abrupt('return');

            case 4:
              state = getState();
              alreadyStartConversation = _get(state, 'behavior.alreadyStartConversation');
              trackingTriggerEvents = _get(state, 'behavior.triggerStartingPointTracking') || [];

              // If enable the pre-chat form, only trigger the startingPoint after starting the conversation.

              if (!(!isDisablePreChatForm(setting) && !alreadyStartConversation)) {
                _context2.next = 10;
                break;
              }

              console.log('ingore the action');
              return _context2.abrupt('return');

            case 10:
              if (!(startingPointType === STARTING_POINT_TYPES.VISITOR_CLICK_ELEMENT)) {
                _context2.next = 14;
                break;
              }

              console.log('triggerTheStartingPoint');
              triggerStartingPoint(startingPointId);
              return _context2.abrupt('return');

            case 14:

              if (startingPointType === STARTING_POINT_TYPES.VISITOR_VISIT_URLS) {
                // console.log('startingPointType === STARTING_POINT_TYPES.VISITOR_VISIT_URLS');
                _ref4 = _find(trackingTriggerEvents, function (event) {
                  return event.startingPointId === startingPointId;
                }) || {}, lastTime = _ref4.lastTime;
                // console.log('lastTime:', lastTime);
                // console.log('isLastTimeMoreThanOneDay: ', isLastTimeMoreThanOneDay(lastTime));

                if (_isEmpty(lastTime) || isLastTimeMoreThanOneDay(lastTime)) {
                  // console.log('trigger');
                  triggerStartingPoint(startingPointId);
                  dispatch(trackingTheTriggerEvent({ startingPointType: startingPointType, startingPointId: startingPointId }));
                }
              }

            case 15:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function (_x2, _x3) {
      return _ref3.apply(this, arguments);
    };
  }();
};

export var triggerTheWelcomeMessageWhenVisitPage = function triggerTheWelcomeMessageWhenVisitPage(setting) {
  return function () {
    var _ref5 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(dispatch, getState) {
      var state, hasTriggeredWelcomeMessage, alreadyStartConversation, delay, _delay;

      return _regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              console.log('triggerTheWelcomeMessageWhenVisitPage');
              state = getState();

              if (setting.isEnableAutomation) {
                _context3.next = 5;
                break;
              }

              console.log('Ingore because not enable bot now');
              return _context3.abrupt('return');

            case 5:
              hasTriggeredWelcomeMessage = _get(state, 'behavior.hasTriggeredWelcomeMessage');
              alreadyStartConversation = _get(state, 'behavior.alreadyStartConversation');

              if (!(!isDisablePreChatForm(setting) && !alreadyStartConversation)) {
                _context3.next = 10;
                break;
              }

              console.log('Ingore because the pre-chat form is enable & the conversation has not started'); // When enable the pre-chat form, only start conversation when click to chat with bot
              return _context3.abrupt('return');

            case 10:
              if (!(!isTriggerWelcomeMessageWhenVisitPage(setting) && !isTriggerWelcomeBackMessageWhenVisitPage(setting))) {
                _context3.next = 13;
                break;
              }

              console.log('Ingore because dont enable welcome message & welcome back message when revisit');
              return _context3.abrupt('return');

            case 13:
              if (!(isTriggerWelcomeMessageWhenVisitPage(setting) && !hasTriggeredWelcomeMessage)) {
                _context3.next = 18;
                break;
              }

              // Trigger the welcome message
              delay = getTimeDuration(_get(setting, 'screenWelcomeMessages.trigger.time'), _get(setting, 'screenWelcomeMessages.trigger.unitOfTime'));

              console.log('...Trigger the welcome message: ', delay);
              setTimeout(function () {
                console.log('Trigger the welcome message');
                sendMessage({
                  group: GROUP.MESSAGE,
                  type: TYPE.MESSAGE.TRIGGER_WELCOME_MESSAGE,
                  payload: {},
                  tenantInfo: getTenantInfo()
                });
                dispatch(updateHasTriggeredWelcomeMessage());
              }, delay);
              return _context3.abrupt('return');

            case 18:
              if (!(hasTriggeredWelcomeMessage && isTriggerWelcomeBackMessageWhenVisitPage(setting))) {
                _context3.next = 23;
                break;
              }

              // Trigger the welcome back message
              _delay = getTimeDuration(_get(setting, 'screenWelcomeBackMessages.trigger.time'), _get(setting, 'screenWelcomeBackMessages.trigger.unitOfTime'));

              console.log('...Trigger the welcome back message:', _delay);
              if (isLastTimeMoreThanAPeriod(_get(state, 'behavior.lastTimeTriggeredWelcomeMessage'), _delay)) {
                console.log('Trigger the welcome back message');
                sendMessage({
                  group: GROUP.MESSAGE,
                  type: TYPE.MESSAGE.TRIGGER_WELCOME_BACK_MESSAGE,
                  payload: {},
                  tenantInfo: getTenantInfo()
                });
                dispatch(updateHasTriggeredWelcomeMessage());
              }
              return _context3.abrupt('return');

            case 23:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    return function (_x4, _x5) {
      return _ref5.apply(this, arguments);
    };
  }();
};

// - Nếu enable pre-chat form (Tức là yêu cầu có thông tin mới bắt đầu chat)
// ---- Nếu in operating hours
// -------- Nếu enable chatbot -> Gửi welcome message.
// ---- Nếu not in operating hours
// -------- Nếu enable automation
// --------- Show chat with bot -> Gửi welcome message.
// -------- Nếu not enable automation
// --------- Show leave message -> Không gửi welcome message.

// - Nếu không enable pre-chat form
// ---- Nếu in operating hours
//
// ---- Nếu not in operating hours