'use strict';

exports.__esModule = true;
exports.retryFuncWithFibonancies = exports.wait = exports.getCurrentScreen = exports.isLastTimeMoreThanAPeriod = exports.isLastTimeMoreThanOneDay = exports.isTriggerWelcomeMessageWhenVisitPage = exports.isTriggerWelcomeBackMessageWhenVisitPage = exports.isTriggerWelcomeMessageWhenClickIcon = exports.isTriggerWelcomeBackMessageWhenClickIcon = exports.isDisablePreChatForm = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isDisablePreChatForm = exports.isDisablePreChatForm = function isDisablePreChatForm(setting) {
  var value = (0, _get3.default)(setting, 'chatboxThemeSettings.preChatForm.enable');
  if (typeof value === 'undefined' || value === false) return true;
  return false;
};

var isTriggerWelcomeBackMessageWhenClickIcon = exports.isTriggerWelcomeBackMessageWhenClickIcon = function isTriggerWelcomeBackMessageWhenClickIcon(setting) {
  return (0, _get3.default)(setting, 'screenWelcomeBackMessages.trigger.triggerType') === 'click-icon';
};

var isTriggerWelcomeMessageWhenClickIcon = exports.isTriggerWelcomeMessageWhenClickIcon = function isTriggerWelcomeMessageWhenClickIcon(setting) {
  var value = (0, _get3.default)(setting, 'screenWelcomeMessages.trigger.triggerType');
  return typeof value === 'undefined' || value === 'click-icon';
};

var isTriggerWelcomeBackMessageWhenVisitPage = exports.isTriggerWelcomeBackMessageWhenVisitPage = function isTriggerWelcomeBackMessageWhenVisitPage(setting) {
  return (0, _get3.default)(setting, 'screenWelcomeBackMessages.trigger.triggerType') === 'visit-page';
};

var isTriggerWelcomeMessageWhenVisitPage = exports.isTriggerWelcomeMessageWhenVisitPage = function isTriggerWelcomeMessageWhenVisitPage(setting) {
  return (0, _get3.default)(setting, 'screenWelcomeMessages.trigger.triggerType') === 'visit-page';
};

var isLastTimeMoreThanOneDay = exports.isLastTimeMoreThanOneDay = function isLastTimeMoreThanOneDay(lastTime) {
  if ((0, _isEmpty3.default)(lastTime)) return false;

  var fromDate = typeof lastTime === 'string' ? new Date(lastTime) : lastTime;
  return Math.abs(fromDate - new Date()) >= 1000 * 60 * 60 * 24;
};

var isLastTimeMoreThanAPeriod = exports.isLastTimeMoreThanAPeriod = function isLastTimeMoreThanAPeriod(lastTime, duration) {
  if (typeof lastTime === 'undefined' || typeof duration === 'undefined') return false;
  var fromDate = typeof lastTime === 'string' ? new Date(lastTime) : lastTime;
  return Math.abs(fromDate - new Date()) >= duration;
};

var DURATION_TO_RESET_DECISION = 30 * 60 * 1000;
// const DURATION_TO_RESET_DECISION = 20 * 1000;

var getCurrentScreen = exports.getCurrentScreen = function getCurrentScreen(props) {
  var isInConfigurationMode = props.isInConfigurationMode,
      isStartConversationFromAdmin = props.isStartConversationFromAdmin,
      demoData = props.demoData,
      setting = props.setting,
      alreadySubmitUserInfo = props.alreadySubmitUserInfo,
      isInOperatingHours = props.isInOperatingHours,
      isEnableAutomation = props.isEnableAutomation,
      lastTimeAcceptChattingWithBot = props.lastTimeAcceptChattingWithBot,
      lastTimeAcceptLeavingOfflineMessage = props.lastTimeAcceptLeavingOfflineMessage,
      lastTimeReceivingMessageFromAdmin = props.lastTimeReceivingMessageFromAdmin;


  if (isInConfigurationMode) {
    return demoData.currentScreen === 'welcome_messages' ? 'conversation' : demoData.currentScreen;
  }

  if (isStartConversationFromAdmin && lastTimeReceivingMessageFromAdmin && !isLastTimeMoreThanAPeriod(lastTimeReceivingMessageFromAdmin, DURATION_TO_RESET_DECISION)) {
    return 'conversation';
  }

  if (!isDisablePreChatForm(setting) && !alreadySubmitUserInfo) {
    return 'form';
  }

  if (!isInOperatingHours) {
    if (isEnableAutomation && (!lastTimeAcceptChattingWithBot || isLastTimeMoreThanAPeriod(lastTimeAcceptChattingWithBot, DURATION_TO_RESET_DECISION))) {
      return 'off_with_chatbot';
    }

    if (!isEnableAutomation && (!lastTimeAcceptLeavingOfflineMessage || isLastTimeMoreThanAPeriod(lastTimeAcceptLeavingOfflineMessage, DURATION_TO_RESET_DECISION))) {
      return 'off';
    }
  }
  return 'conversation';
};

var wait = exports.wait = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(duration) {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            return _context.abrupt('return', new Promise(function (resolve, reject) {
              setTimeout(resolve, duration);
            }));

          case 1:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function wait(_x) {
    return _ref.apply(this, arguments);
  };
}();

var retryFuncWithFibonancies = exports.retryFuncWithFibonancies = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(funcChecking, fiboNumbers) {
    var fibonanciNumbers, currentWaitTimeIndex;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            fibonanciNumbers = fiboNumbers || [0.5, 1, 2, 3, 5, 8, 13];
            currentWaitTimeIndex = 0;

          case 2:
            _context2.next = 4;
            return wait(fibonanciNumbers[currentWaitTimeIndex] * 1000);

          case 4:
            ++currentWaitTimeIndex;

          case 5:
            if (!funcChecking() && currentWaitTimeIndex < fibonanciNumbers.length - 1) {
              _context2.next = 2;
              break;
            }

          case 6:
            if (funcChecking()) {
              _context2.next = 8;
              break;
            }

            throw new Error();

          case 8:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function retryFuncWithFibonancies(_x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();