'use strict';

exports.__esModule = true;

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _includes = require('lodash/includes');

var _includes2 = _interopRequireDefault(_includes);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _commonStyles = require('../../common-styles.scss');

var _commonStyles2 = _interopRequireDefault(_commonStyles);

var _utils = require('../../../../commons/utils');

var _reactRedux = require('react-redux');

var _actions = require('../../../../store/actions');

var _widgetContext = require('../../../../widget-context');

var _widgetContext2 = _interopRequireDefault(_widgetContext);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var UserInfoForm = (0, _react.lazy)(function () {
  return import('../../user-info-form');
});
var Header = (0, _react.lazy)(function () {
  return import('../../header');
});
var Power = (0, _react.lazy)(function () {
  return import('../../power');
});
var ChatBoxInput = (0, _react.lazy)(function () {
  return import('../../input');
});
var OfflineStatus = (0, _react.lazy)(function () {
  return import('../../offline');
});
var ChatMessageContainer = (0, _react.lazy)(function () {
  return import('../message-container');
});
var NoConect = (0, _react.lazy)(function () {
  return import('../../no-connection');
});

exports.default = function (props) {
  var isInConfigurationMode = props.isInConfigurationMode,
      isMobileMode = props.isMobileMode,
      setting = props.setting,
      messages = props.messages,
      participants = props.participants,
      demoData = props.demoData;

  var dispatch = (0, _reactRedux.useDispatch)();

  var _useContext = (0, _react.useContext)(_widgetContext2.default),
      isSimulator = _useContext.isSimulator;

  var showedScreen = isSimulator ? 'conversation' : (0, _utils.getCurrentScreen)(props);
  var soundOn = isSimulator ? false : (0, _reactRedux.useSelector)(function (store) {
    return (0, _get3.default)(store, 'behavior.soundOn');
  });
  var isViewForOffline = showedScreen === 'off' || showedScreen === 'off_with_chatbot';
  return _react2.default.createElement(
    'div',
    { className: _commonStyles2.default.conversationContainer },
    isViewForOffline && _react2.default.createElement(
      _react.Suspense,
      { fallback: _react2.default.createElement('div', null) },
      _react2.default.createElement(OfflineStatus, { setting: setting, isInConfigurationMode: isInConfigurationMode, showedScreen: showedScreen })
    ),
    !isViewForOffline && _react2.default.createElement(
      _react2.default.Fragment,
      null,
      !isSimulator && !(0, _includes2.default)(['off', 'no_connection'], showedScreen) && _react2.default.createElement(
        _react.Suspense,
        { fallback: _react2.default.createElement('div', null) },
        _react2.default.createElement(Header, { setting: setting, isInConfigurationMode: isInConfigurationMode, demoData: demoData, currentScreen: showedScreen })
      ),
      _react2.default.createElement(
        'div',
        { className: _commonStyles2.default.boxBody },
        showedScreen === 'form' && _react2.default.createElement(
          _react.Suspense,
          { fallback: _react2.default.createElement('div', null) },
          _react2.default.createElement(UserInfoForm, { setting: setting, isInConfigurationMode: isInConfigurationMode })
        ),
        showedScreen === 'conversation' && _react2.default.createElement(
          'div',
          { className: _commonStyles2.default.messageContainer },
          _react2.default.createElement(
            _react.Suspense,
            { fallback: _react2.default.createElement('div', null) },
            _react2.default.createElement(ChatMessageContainer, {
              containerBgColor: 'white',
              isShowLeftAvatar: true,
              isShowRightAvatar: false,
              isShowSeenTime: false,
              avatarWidth: 25,
              avatarMargin: 0,
              style: 1,
              textFontSize: 12,
              messageMargin: 25,
              onScrollTop: function onScrollTop() {},
              onClickButton: function onClickButton(buttonData) {
                return dispatch((0, _actions.sendButtonMessage)(buttonData, setting.channelId));
              },
              leftParitcipantsTextColor: (0, _get3.default)(setting, 'screenConversation.leftSideTextColor'),
              leftParitcipantsTextBgColor: (0, _get3.default)(setting, 'screenConversation.leftSideBackgroundColor'),
              rightParitcipantsTextColor: (0, _get3.default)(setting, 'screenConversation.rightSideTextColor'),
              rightParitcipantsTextBgColor: (0, _get3.default)(setting, 'screenConversation.rightSideBackgroundColor'),
              messages: isInConfigurationMode ? demoData.messages : messages,
              participants: isInConfigurationMode ? demoData.participants : participants,
              soundOn: soundOn,
              setting: setting
            })
          )
        ),
        showedScreen === 'conversation' && _react2.default.createElement(
          _react.Suspense,
          { fallback: _react2.default.createElement('div', null) },
          _react2.default.createElement(ChatBoxInput, { setting: setting, isInConfigurationMode: isInConfigurationMode })
        ),
        showedScreen === 'no_connection' && _react2.default.createElement(
          _react.Suspense,
          { fallback: _react2.default.createElement('div', null) },
          _react2.default.createElement(NoConect, { setting: setting, isInConfigurationMode: isInConfigurationMode, demoData: demoData, isMobileMode: isMobileMode })
        )
      )
    ),
    _react2.default.createElement(Power, { setting: setting, showedScreen: showedScreen })
  );
};

module.exports = exports['default'];