'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _size2 = require('lodash/size');

var _size3 = _interopRequireDefault(_size2);

var _filter2 = require('lodash/filter');

var _filter3 = _interopRequireDefault(_filter2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactRedux = require('react-redux');

var _redux = require('redux');

var _demoScreenTypes = require('../../commons/constants/demo-screen-types');

var _reactResizeObserver = require('react-resize-observer');

var _reactResizeObserver2 = _interopRequireDefault(_reactResizeObserver);

var _actions = require('../../store/actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ChatBox = (0, _react.lazy)(function () {
  return import('../chat-box');
});
var PhoneCall = (0, _react.lazy)(function () {
  return import('../phone-call');
});
var ScrollTop = (0, _react.lazy)(function () {
  return import('../scroll-top');
});
var Messenger = (0, _react.lazy)(function () {
  return import('../messenger');
});
var Support = (0, _react.lazy)(function () {
  return import('../support');
});
var Zalo = (0, _react.lazy)(function () {
  return import('../zalo');
});
var Instagram = (0, _react.lazy)(function () {
  return import('../instagram');
});
var Email = (0, _react.lazy)(function () {
  return import('../email');
});
var LinkedIn = (0, _react.lazy)(function () {
  return import('../linkedIn');
});
var Skype = (0, _react.lazy)(function () {
  return import('../skype');
});
var Sms = (0, _react.lazy)(function () {
  return import('../sms');
});
var Twitter = (0, _react.lazy)(function () {
  return import('../twitter');
});
var Viber = (0, _react.lazy)(function () {
  return import('../viber');
});
var Youtube = (0, _react.lazy)(function () {
  return import('../youtube');
});
var Tiktok = (0, _react.lazy)(function () {
  return import('../tiktok');
});

// import "antd/dist/antd.css";
var POSITION_LEFT_BOTTOM = 'left-bottom';
var VERTICAL = 'vertical';
var HORIZONTAL = 'horizontal';
var MOBILE_WIDTH = 767;
var iconMargin = 10;
// const iconWidth = 60;
var boostcrmWidgets = ['chatbox', 'support', 'scrollTop', 'call'];
var socialWidgets = ['email', 'instagram', 'linkedin', 'skype', 'sms', 'twitter', 'viber', 'youtube', 'messenger', 'zalo', 'tiktok'];
function isInLeftBottom(component) {
  if ((0, _get3.default)(component, 'position') === POSITION_LEFT_BOTTOM) {
    return true;
  }
  return false;
}

var getSvgSize = function getSvgSize(size) {
  var svgSize = 33;
  if (size === 50) {
    svgSize = svgSize - 5;
    return svgSize;
  }
  if (size === 70) {
    svgSize = svgSize + 5;
    return svgSize;
  }
  return svgSize;
};

var calculateMargin = function calculateMargin(rect, iconWidth, value, unit, isWidth) {
  switch (unit) {
    case 'em':
      return value * iconWidth || 10;
    case '%':
      return value * (isWidth ? rect.width : rect.height) / 100 || 10;
    case 'px':
    default:
      return value || 10;
  }
};

var display = {
  COLLAPSED: 'COLLAPSED',
  EXPANDED: 'EXPANDED'
};

var direction = {
  VERTICAL: 'VERTICAL',
  HORIZONTAL: 'HORIZONTAL'
};

var positionWidget = {
  RIGHT_BOTTOM: 'RIGHT_BOTTOM',
  LEFT_BOTTOM: 'LEFT_BOTTOM'
};

var supportComponent = {
  configurationData: {},
  enable: true,
  order: 99,
  position: 'right-bottom'
};

var getInitialStateFromProps = function getInitialStateFromProps(props) {
  var isFixedRect = props.isFixedRect,
      rect = props.rect,
      setting = props.setting,
      demoData = props.demoData,
      isInConfigurationMode = props.isInConfigurationMode,
      showSupportWidget = props.showSupportWidget;

  var _ref = demoData || {},
      currentComponent = _ref.currentComponent;

  var margins = setting.margins;

  var defaultDisplay = (0, _get3.default)(setting, 'display.style');

  var bottom = margins.bottom,
      left = margins.left,
      right = margins.right;

  var iconWidth = (0, _get3.default)(setting, 'iconSize');
  var marginRight = calculateMargin(rect, iconWidth, (0, _get3.default)(right, 'value'), (0, _get3.default)(right, 'cssUnit'), true);
  var marginLeft = calculateMargin(rect, iconWidth, (0, _get3.default)(left, 'value'), (0, _get3.default)(left, 'cssUnit'), true);
  var marginBottom = calculateMargin(rect, iconWidth, (0, _get3.default)(bottom, 'value'), (0, _get3.default)(bottom, 'cssUnit'), false);
  return (0, _extends3.default)({
    setting: setting,
    demoData: demoData,
    isSkipHandlingCollapseExpandEvent: isInConfigurationMode && currentComponent !== _demoScreenTypes.DEMO_SCREEN_TYPES.GENERAL,
    isSettingCollapse: defaultDisplay === display.COLLAPSED,
    isCurrentExpand: isInConfigurationMode && currentComponent !== _demoScreenTypes.DEMO_SCREEN_TYPES.GENERAL || showSupportWidget,
    isSettingLeftBottom: (0, _get3.default)(setting, 'display.position') === positionWidget.LEFT_BOTTOM,
    isSettingVertical: (0, _get3.default)(setting, 'display.direction') === direction.VERTICAL,
    isEnableSocialWidgets: (0, _get3.default)(setting, 'enableSocialWidgets'),
    marginRight: marginRight,
    marginLeft: marginLeft,
    marginBottom: marginBottom
  }, isFixedRect ? {
    rect: rect,
    isMobileMode: rect.width < MOBILE_WIDTH
  } : {});
};

var Widget = function (_Component) {
  (0, _inherits3.default)(Widget, _Component);

  function Widget() {
    var _temp, _this, _ret;

    (0, _classCallCheck3.default)(this, Widget);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, _Component.call.apply(_Component, [this].concat(args))), _this), _this.state = {
      isMobileMode: false,
      rect: {
        width: 0,
        height: 0
      }
    }, _this.toggleCollapseExpandWidget = function () {
      if (_this.state.isSkipHandlingCollapseExpandEvent) {
        return;
      }

      if (_this.props.isInConfigurationMode) {
        _this.setState({
          isCurrentExpand: !_this.state.isCurrentExpand
        });
        return;
      }

      _this.props.actions.toggleTheSupportWidgetRequest(!_this.state.isCurrentExpand);
    }, _this.getWidgetComponent = function (name) {
      var _this$props$setting = _this.props.setting,
          chatboxComponent = _this$props$setting.chatboxComponent,
          messengerComponent = _this$props$setting.messengerComponent,
          zaloComponent = _this$props$setting.zaloComponent,
          callComponent = _this$props$setting.callComponent,
          scrollTopComponent = _this$props$setting.scrollTopComponent,
          tempWidgets = _this$props$setting.tempWidgets;

      var widgets = (0, _filter3.default)(tempWidgets, function (widget) {
        return widget.type.indexOf(name) !== -1;
      });
      if ((0, _size3.default)(widgets) > 0) {
        widgets = widgets.map(function (widget) {
          return { name: name, component: widget };
        });
      }
      switch (name) {
        case 'chatbox':
          return [{ name: name, component: chatboxComponent }];
        case 'call':
          return [{ name: name, component: callComponent }];
        case 'scrollTop':
          return [{ name: name, component: (0, _extends3.default)({}, scrollTopComponent, { order: 0 }) }];
        case 'support':
          return [{ name: name, component: supportComponent }];
        case 'zalo':
          zaloComponent && widgets.push({ name: name, component: (0, _extends3.default)({}, zaloComponent, { type: 'zalo_legacy' }) });
          return widgets;
        case 'messenger':
          messengerComponent && widgets.push({ name: name, component: (0, _extends3.default)({}, messengerComponent, { type: 'messenger_legacy' }) });
          return widgets;
        case 'email':
        case 'instagram':
        case 'linkedin':
        case 'skype':
        case 'sms':
        case 'twitter':
        case 'youtube':
        case 'viber':
        case 'tiktok':
          return widgets;
      }
    }, _this.calculateAmountWidget = function (widgets) {
      widgets = widgets.filter(function (widget) {
        return widget.name !== 'support' && widget.name !== 'scrollTop' && (0, _get3.default)(widget, 'component.enable') === true;
      });
      return widgets.length;
    }, _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
  }

  Widget.prototype.componentDidMount = function componentDidMount() {
    this.setState(getInitialStateFromProps(this.props));
  };

  Widget.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
    if (props.setting !== state.setting || props.demoData !== state.demoData) {
      return getInitialStateFromProps(props);
    }

    if (!props.isInConfigurationMode && props.showSupportWidget !== state.isCurrentExpand) {
      return {
        isCurrentExpand: props.showSupportWidget
      };
    }
    return null;
  };

  Widget.prototype.getComponentPosition = function getComponentPosition() {
    var _this2 = this;

    var _state = this.state,
        isSettingCollapse = _state.isSettingCollapse,
        isCurrentExpand = _state.isCurrentExpand,
        rect = _state.rect,
        isEnableSocialWidgets = _state.isEnableSocialWidgets,
        marginRight = _state.marginRight,
        marginLeft = _state.marginLeft,
        marginBottom = _state.marginBottom;


    var iconWidth = (0, _get3.default)(this.props.setting, 'iconSize');
    var leftBottomList = [];
    var rightBottomList = [];
    var isLeftBottom = (0, _get3.default)(this.props.setting, 'display.position') === positionWidget.LEFT_BOTTOM;
    var isVertical = (0, _get3.default)(this.props.setting, 'display.direction') === direction.VERTICAL;

    var widgets = [].concat(boostcrmWidgets, socialWidgets);

    if (isSettingCollapse) {
      if (isLeftBottom) {
        widgets.forEach(function (name) {
          leftBottomList = [].concat(leftBottomList, _this2.getWidgetComponent(name));
        });
      } else {
        widgets.forEach(function (name) {
          rightBottomList = [].concat(rightBottomList, _this2.getWidgetComponent(name));
        });
      }
    } else {
      widgets.filter(function (name) {
        return name !== 'support';
      }).forEach(function (name) {
        if (isLeftBottom) {
          leftBottomList = [].concat(leftBottomList, _this2.getWidgetComponent(name));
        } else {
          rightBottomList = [].concat(rightBottomList, _this2.getWidgetComponent(name));
        }
      });
    }

    var amountWidget = this.calculateAmountWidget([].concat(leftBottomList, rightBottomList));

    if (amountWidget <= 1) {
      isSettingCollapse = false;
    }

    if (amountWidget > 6) {
      if (!isSettingCollapse) {
        if (isLeftBottom) {
          leftBottomList = [].concat(leftBottomList, [{ name: 'support', component: supportComponent }]);
        } else {
          rightBottomList = [].concat(rightBottomList, [{ name: 'support', component: supportComponent }]);
        }
      }
      isSettingCollapse = true;
    }

    if (isSettingCollapse) {
      leftBottomList = leftBottomList.filter(function (component) {
        return (0, _get3.default)(component, 'component.enable') === true && (isCurrentExpand || (0, _get3.default)(component, 'name') === 'support' || (0, _get3.default)(component, 'name') === 'scrollTop');
      }).sort(function (a, b) {
        return (0, _get3.default)(b, 'component.order') - (0, _get3.default)(a, 'component.order');
      });

      rightBottomList = rightBottomList.filter(function (component) {
        return (0, _get3.default)(component, 'component.enable') === true && (isCurrentExpand || (0, _get3.default)(component, 'name') === 'support' || (0, _get3.default)(component, 'name') === 'scrollTop');
      }).sort(function (a, b) {
        return (0, _get3.default)(b, 'component.order') - (0, _get3.default)(a, 'component.order');
      });
    } else {
      leftBottomList = leftBottomList.filter(function (component) {
        return (0, _get3.default)(component, 'component.enable') === true;
      }).sort(function (a, b) {
        return (0, _get3.default)(b, 'component.order') - (0, _get3.default)(a, 'component.order');
      });

      rightBottomList = rightBottomList.filter(function (component) {
        return (0, _get3.default)(component, 'component.enable') === true;
      }).sort(function (a, b) {
        return (0, _get3.default)(b, 'component.order') - (0, _get3.default)(a, 'component.order');
      });
    }
    var result = [];

    if (amountWidget <= 1) {
      leftBottomList = leftBottomList.filter(function (widget) {
        return widget.name !== 'support';
      });
      rightBottomList = rightBottomList.filter(function (widget) {
        return widget.name !== 'support';
      });
    }

    if (isSettingCollapse && amountWidget > 1) {
      leftBottomList.forEach(function (component, idx) {
        result.push({
          enable: true,
          position: isVertical || component.name === 'scrollTop' ? {
            right: rect.width - iconWidth - marginLeft,
            bottom: (iconWidth + iconMargin) * (!isVertical ? 1 : idx) + marginBottom
          } : {
            right: idx === 0 ? rect.width - (marginLeft + iconWidth) : rect.width - ((iconMargin + iconWidth) * idx + marginLeft + iconWidth),
            bottom: marginBottom
          },
          area: 'left-bottom',
          type: component.name,
          component: component.component
        });
      });
      rightBottomList.forEach(function (component, idx) {
        result.push({
          enable: true,
          position: isVertical || component.name === 'scrollTop' ? {
            right: marginRight,
            bottom: (iconWidth + iconMargin) * (!isVertical ? 1 : idx) + marginBottom
          } : {
            right: (iconWidth + iconMargin) * idx + marginRight,
            bottom: marginBottom
          },
          area: 'right-bottom',
          type: component.name,
          component: component.component
        });
      });
    } else {
      leftBottomList.forEach(function (component, idx) {
        result.push({
          enable: true,
          position: isVertical ? {
            right: rect.width - iconWidth - marginLeft,
            bottom: (iconWidth + iconMargin) * idx + marginBottom
          } : {
            right: idx === 0 ? rect.width - (marginLeft + iconWidth) : rect.width - ((iconMargin + iconWidth) * idx + marginLeft + iconWidth),
            bottom: marginBottom
          },
          area: 'left-bottom',
          type: component.name,
          component: component.component
        });
      });

      rightBottomList.forEach(function (component, idx) {
        result.push({
          enable: true,
          position: isVertical ? {
            right: marginRight,
            bottom: (iconWidth + iconMargin) * idx + marginBottom
          } : {
            right: (iconWidth + iconMargin) * idx + marginRight,
            bottom: marginBottom
          },
          area: 'right-bottom',
          type: component.name,
          component: component.component
        });
      });
    }

    return result;
  };

  Widget.prototype.render = function render() {
    var _this3 = this;

    var widgets = this.getComponentPosition();
    var isFixedRect = this.props.isFixedRect;

    var iconSize = (0, _get3.default)(this.props.setting, 'iconSize');
    var svgSize = getSvgSize(iconSize);
    return _react2.default.createElement(
      'div',
      { style: { width: '100%', height: '100%', position: 'relative' } },
      !isFixedRect && _react2.default.createElement(_reactResizeObserver2.default, {
        onPosition: function onPosition(rect) {
          _this3.setState({
            rect: {
              height: rect.top,
              width: rect.left
            },
            isMobileMode: rect.left <= MOBILE_WIDTH
          });
        }
      }),
      (widgets || []).map(function (widget, index) {
        switch (widget.type) {
          case 'scrollTop':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(ScrollTop, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'call':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(PhoneCall, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'chatbox':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(ChatBox, (0, _extends3.default)({ index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'zalo':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Zalo, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'messenger':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Messenger, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'email':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Email, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'linkedin':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(LinkedIn, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'skype':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Skype, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'sms':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Sms, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'twitter':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Twitter, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'viber':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Viber, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'youtube':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Youtube, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'instagram':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Instagram, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'tiktok':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Tiktok, (0, _extends3.default)({ key: index, index: index }, _this3.props, _this3.state, widget, { svgSize: svgSize }))
            );
          case 'support':
            return _react2.default.createElement(
              _react.Suspense,
              { key: index, index: index, fallback: _react2.default.createElement('div', null) },
              _react2.default.createElement(Support, (0, _extends3.default)({
                key: index
              }, _this3.props, _this3.state, widget, {
                svgSize: svgSize,
                isCurrentExpand: _this3.state.isCurrentExpand,
                toggleCollapseExpandWidget: _this3.toggleCollapseExpandWidget
              }))
            );
        }
      })
    );
  };

  return Widget;
}(_react.Component);

exports.default = (0, _reactRedux.connect)(function (store) {
  return {
    showChat: (0, _get3.default)(store, 'behavior.showChat'),
    showSupportWidget: (0, _get3.default)(store, 'behavior.showSupportWidget'),
    numberOfNewMessages: (0, _get3.default)(store, 'messages.numberOfNewMessages')
  };
}, function (dispatch) {
  return {
    actions: (0, _redux.bindActionCreators)({
      toggleTheSupportWidgetRequest: _actions.toggleTheSupportWidgetRequest
    }, dispatch)
  };
})(Widget);
module.exports = exports['default'];