'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _debounce2 = require('lodash/debounce');

var _debounce3 = _interopRequireDefault(_debounce2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

exports.initWSClient = initWSClient;
exports.closeWSClient = closeWSClient;
exports.disconnectWSClient = disconnectWSClient;
exports.reconnectWSClient = reconnectWSClient;
exports.sendMessage = sendMessage;
exports.sendWebsiteEvent = sendWebsiteEvent;
exports.triggerStartingPoint = triggerStartingPoint;
exports.setMessageHanlder = setMessageHanlder;
exports.setStartConversationHandler = setStartConversationHandler;
exports.setResumeConversationHandler = setResumeConversationHandler;

var _socket = require('socket.io-client');

var io = _interopRequireWildcard(_socket);

var _urlJoin = require('url-join');

var _urlJoin2 = _interopRequireDefault(_urlJoin);

var _localStorage = require('../commons/utils/local-storage');

var _localStorage2 = require('../commons/constants/local-storage');

var _tenantInfo = require('../commons/tenant-info');

var _getClientInfo = require('../commons/utils/get-client-info');

var _getClientInfo2 = _interopRequireDefault(_getClientInfo);

var _websocket = require('../commons/constants/websocket');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var queuedMessages = [];

var wsClient = null;
var getWSHost = function getWSHost(apiHost) {
  if (apiHost.indexOf('localhost') >= 0 || apiHost.indexOf('127.0.0.1') >= 0) return apiHost;
  return apiHost.replace('https://', 'https://ws.').replace('http://', 'http://ws.');
};

var sendOrQueueMessage = function sendOrQueueMessage(wsMessageType, payload) {
  // console.log(`wsMessageType ${wsMessageType}:`, JSON.stringify(payload));
  var tenantInfo = (0, _tenantInfo.getTenantInfo)();
  // console.log("tenantInfo:", JSON.stringify(tenantInfo));
  if (wsClient && wsClient.socket && wsClient.socket.connected && tenantInfo.conversationId) {
    // Send all messages in queue first if available
    // console.log("Send all messages in queue");
    while (queuedMessages.length > 0) {
      console.log('send message from queue');
      var queuedMessage = queuedMessages.shift();
      queuedMessage && wsClient.emitMessage(queuedMessage.wsMessageType, (0, _extends3.default)({}, queuedMessage.payload, {
        tenantInfo: tenantInfo
      }));
    }

    wsClient.emitMessage(wsMessageType, (0, _extends3.default)({}, payload, {
      tenantInfo: tenantInfo
    }));
  } else {
    console.log('enqueue the message: ', { wsMessageType: wsMessageType, payload: payload });
    // Add to queue
    queuedMessages.push({ wsMessageType: wsMessageType, payload: payload });
    // Enable retry
    retrySendQueuedMessages();
  }
};

var retrySendQueuedMessages = function retrySendQueuedMessages(channelId) {
  console.log('retrySendQueuedMessages');
  var interval = setInterval(function () {
    var tenantInfo = (0, _tenantInfo.getTenantInfo)();
    if (wsClient && wsClient.connected && tenantInfo.conversationId) {
      console.log('send messsages from queues: ', queuedMessages.length);
      while (queuedMessages.length > 0) {
        var queuedMessage = queuedMessages.shift();
        queuedMessage && wsClient.emitMessage(queuedMessage.wsMessageType, (0, _extends3.default)({}, queuedMessage.payload, {
          tenantInfo: tenantInfo
        }));
      }
      clearInterval(interval);
    }
  }, 1000);
};

var debounceReconnect = (0, _debounce3.default)(function (ws, onConnected) {
  ws.connected && onConnected && onConnected();
}, 3000);

var WebSocket = function () {
  function WebSocket(host, token, isFromSimulator, channelKey) {
    var _this = this;

    (0, _classCallCheck3.default)(this, WebSocket);

    this.onMessageHandler = null;
    this.onInitWSSuccess = null;
    this.onStartConversationSuccess = null;
    this.isConnected = false; // Important! Don't depend on `wsClient.socket.connected`. This field could be true before the event onConnect
    this.isReconnecting = false;
    var WS_HOST = (0, _urlJoin2.default)(getWSHost(host), '/chatbox');
    this.socket = io(WS_HOST, {
      transports: ['websocket', 'polling'],
      query: {
        token: '' + token
      },
      forceNew: true,
      reconnection: true,
      reconnectionDelay: 2000,
      reconnectionDelayMax: 5000,
      reconnectionAttempts: 10,
      timeout: 30000
    });

    this.socket.on('connect', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
      var channelId, tenantId, conversationId;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              console.log('on connect');
              _this.connected = true;
              // Check if it is a old conversation
              channelId = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.CHANNEL_ID, channelKey));
              tenantId = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.TENANT_ID, channelKey));
              conversationId = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey));

              if (!(isFromSimulator || !conversationId || conversationId === 'undefined')) {
                _context.next = 18;
                break;
              }

              _context.t0 = _this.socket;
              _context.t1 = channelId;
              _context.t2 = tenantId;
              _context.next = 11;
              return (0, _getClientInfo2.default)();

            case 11:
              _context.t3 = _context.sent;
              _context.t4 = isFromSimulator;
              _context.t5 = {
                channelId: _context.t1,
                tenantId: _context.t2,
                visitor: _context.t3,
                isFromSimulator: _context.t4
              };

              _context.t6 = function (response) {
                conversationId = (0, _get3.default)(response, 'id');
                (0, _localStorage.setLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey), conversationId);

                _this.onStartConversationSuccess && _this.onStartConversationSuccess(conversationId);
              };

              _context.t0.emit.call(_context.t0, 'new-conversation', _context.t5, _context.t6);

              _context.next = 28;
              break;

            case 18:
              _context.t7 = _this.socket;
              _context.t8 = channelId;
              _context.t9 = tenantId;
              _context.t10 = conversationId;
              _context.next = 24;
              return (0, _getClientInfo2.default)();

            case 24:
              _context.t11 = _context.sent;
              _context.t12 = {
                channelId: _context.t8,
                tenantId: _context.t9,
                conversationId: _context.t10,
                visitor: _context.t11
              };

              _context.t13 = function (response) {
                // console.log("response: ", response);
                conversationId = (0, _get3.default)(response, 'id');
                if (conversationId) {
                  (0, _localStorage.setLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey), conversationId);
                  _this.onStartConversationSuccess && _this.onStartConversationSuccess(conversationId);
                }
              };

              _context.t7.emit.call(_context.t7, 'old-conversation', _context.t12, _context.t13);

            case 28:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    })));

    this.socket.on('error', function (err) {
      console.log('error on socket:', err);
    });

    this.socket.on('disconnect', function (reason) {
      _this.connected = false;
      console.log('WS:socket disconnect:', reason);
      if (reason === 'io server disconnect') {
        _this.socket.connect();
        return;
      }
    });

    this.socket.on('message', function (data) {
      _this.onMessageHandler && _this.onMessageHandler(data);
    });

    this.socket.on('ping', function (data) {
      if (data === 'ping') {
        _this.emitMessage('message', 'pong');
      }
    });

    this.socket.on('reconnecting', function (attemptNumber) {
      console.log('WS:on reconnecting, total attemptNumber: ', attemptNumber);
      _this.onReconnecting && _this.onReconnecting();
    });

    this.socket.on('reconnect_error', function (error) {
      console.log('WS:on reconnect_error: ');
    });

    this.socket.on('reconnect_failed', function () {
      _this.connected = false;
      console.log('WS:on reconnect_failed: ');
      _this.onConnectFailed && _this.onConnectFailed();
    });

    this.socket.on('connect_error', function (error) {
      _this.connected = false;
      console.log('WS:on connect_error: ');
    });

    this.socket.on('connect_timeout', function (timeout) {
      console.log('WS:on connect_timeout: ', timeout);
      _this.onConnectFailed && _this.onConnectFailed();
    });

    this.socket.on('reconnect', function (attemptNumber) {
      _this.connected = true;
      console.log('WS:on reconnect after attemptNumber: ', attemptNumber);
      debounceReconnect(_this.socket, _this.onConnected);
    });
  }

  WebSocket.prototype.emitMessage = function emitMessage(type, data) {
    if (!this.connected) {
      console.log('emit message when no connection: ', data);
      // Debug root cause show the connection interrupted
      // this.onConnectFailed && this.onConnectFailed();
      return;
    }
    this.socket.emit(type, data);
  };

  WebSocket.prototype.onReceiveMessage = function onReceiveMessage(handler) {
    // console.log("set onReceiveMessageHandler");
    this.onMessageHandler = handler;
  };

  WebSocket.prototype.onStartConversation = function onStartConversation(handler) {
    this.onStartConversationHandler = handler;
  };

  WebSocket.prototype.onResumeConversation = function onResumeConversation(handler) {
    this.onResumeConversationHandler = handler;
  };

  WebSocket.prototype.closeConnection = function closeConnection() {
    if (this.socket && this.socket.close) {
      this.socket.close();
    }
  };

  WebSocket.prototype.disconnect = function disconnect() {
    this.socket.disconnect();
    this.onConnectFailed && this.onConnectFailed();
  };

  WebSocket.prototype.connect = function connect() {
    this.socket.connect();
    this.onConnected && this.onConnected();
  };

  WebSocket.prototype.isWSConnected = function isWSConnected() {
    return this.socket.connected;
  };

  return WebSocket;
}();

function initWSClient(host, token, channelId, isFromSimulator, onInitWSSuccess, onStartConversationSuccess, onConnectFailed, onReconnecting, onConnected) {
  // console.log("initWSClient");
  if (wsClient) return wsClient;
  wsClient = new WebSocket(host, token, isFromSimulator, channelId);
  wsClient.onStartConversationSuccess = onStartConversationSuccess;
  wsClient.onInitWSSuccess = onInitWSSuccess;
  wsClient.onConnectFailed = onConnectFailed;
  wsClient.onReconnecting = onReconnecting;
  wsClient.onConnected = onConnected;
  return wsClient;
}

function closeWSClient() {
  if (wsClient) {
    wsClient.closeConnection();
    wsClient = null;
  }
}

function disconnectWSClient() {
  if (!wsClient) return null;
  wsClient.disconnect();
}

function reconnectWSClient() {
  if (!wsClient) return null;
  !wsClient.isWSConnected() && wsClient.connect();
}

function sendMessage(message) {
  sendOrQueueMessage('message', message);
}

function sendWebsiteEvent(eventType, payload) {
  sendOrQueueMessage('website-event', {
    event: { type: eventType, payload: payload }
  });
}

function triggerStartingPoint(id) {
  sendOrQueueMessage('message', {
    group: _websocket.GROUP.MESSAGE,
    type: _websocket.TYPE.MESSAGE.TRIGGER_STARTING_POINT,
    payload: {
      id: id
    }
  });
}

function setMessageHanlder(handler) {
  if (!wsClient) {
    // console.log("Could not set message handler");
    return;
  }
  wsClient.onReceiveMessage(handler);
}

function setStartConversationHandler(handler) {
  if (!wsClient) return;
  wsClient.onStartConversationHandler(handler);
}

function setResumeConversationHandler(handler) {
  if (!wsClient) return;
  wsClient.onResumeConversationHandler(handler);
}