'use strict';

exports.__esModule = true;
exports.resetNumberOfNewMessages = exports.getMessages = exports.getMessagesError = exports.getMessagesSuccess = exports.getMessagesRequest = exports.sendButtonMessage = exports.sendAttachment = exports.sendAttachmentError = exports.sendAttachmentSuccess = exports.sendAttachmentRequest = exports.sendTextMessage = exports.sendTextMessageSuccess = exports.sendTextMessageRequest = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _join2 = require('lodash/join');

var _join3 = _interopRequireDefault(_join2);

var _size2 = require('lodash/size');

var _size3 = _interopRequireDefault(_size2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _includes2 = require('lodash/includes');

var _includes3 = _interopRequireDefault(_includes2);

var _actionType = require('./action-type');

var actions = _interopRequireWildcard(_actionType);

var _reduxActions = require('redux-actions');

var _websocket = require('../../connections/websocket');

var _websocket2 = require('../../commons/constants/websocket');

var _messageType = require('../../commons/constants/message-type');

var _tenantInfo = require('../../commons/tenant-info');

var _formData = require('form-data');

var FormData = _interopRequireWildcard(_formData);

var _localStorage = require('../../commons/utils/local-storage');

var _localStorage2 = require('../../commons/constants/local-storage');

var _messageStatus = require('../../commons/constants/message-status');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var sendTextMessageRequest = exports.sendTextMessageRequest = (0, _reduxActions.createAction)(actions.SEND_TEXT_MESSAGE + '.REQUEST');
var sendTextMessageSuccess = exports.sendTextMessageSuccess = (0, _reduxActions.createAction)(actions.SEND_TEXT_MESSAGE + '.SUCCESS');
var sendTextMessage = exports.sendTextMessage = function sendTextMessage(text) {
  return function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(dispatch, store) {
      var reqPayload;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              // console.log("sendTextMessage:", text, getTenantInfo());
              reqPayload = { type: _messageType.MESSAGE_TYPES.TEXT, content: { text: text }, externalId: new Date().getTime() };
              _context.next = 3;
              return dispatch(sendTextMessageRequest(reqPayload));

            case 3:
              (0, _websocket.sendMessage)({
                group: _websocket2.GROUP.MESSAGE,
                type: _websocket2.TYPE.MESSAGE.NEW_MESSAGE_FROM_CHATBOX,
                payload: reqPayload,
                tenantInfo: (0, _tenantInfo.getTenantInfo)()
              });

            case 4:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }();
};

var sendAttachmentRequest = exports.sendAttachmentRequest = (0, _reduxActions.createAction)(actions.SEND_ATTACHMENT + '.REQUEST');

var sendAttachmentSuccess = exports.sendAttachmentSuccess = (0, _reduxActions.createAction)(actions.SEND_ATTACHMENT + '.SUCCESS');

var sendAttachmentError = exports.sendAttachmentError = (0, _reduxActions.createAction)(actions.SEND_ATTACHMENT + '.ERROR');

var sendAttachment = exports.sendAttachment = function sendAttachment(text, files, channelKey) {
  return function (dispatch, store) {
    // console.log("sendAttachment.text: ", text);
    // console.log("sendAttachment.files: ", files);
    var isSimulator = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.IS_SIMULATOR, channelKey));
    // Get tenantId ,conversationId, channelId
    var channelId = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.CHANNEL_ID, channelKey));
    var tenantId = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.TENANT_ID, channelKey));
    var conversationId = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey));
    // console.log("sendAttachment", channelId, tenantId, conversationId);
    var form = new FormData();
    form.append('tenantId', tenantId);
    form.append('channelId', channelId);
    form.append('conversationId', conversationId);

    var listUUIDOrder = [];
    files.map(function (file) {
      form.append('files', file.originalFile);

      var reqPayload = void 0;
      var isImage = (0, _get3.default)(file, 'isImage');
      var externalId = new Date().getTime();
      // console.log(`externalId`, externalId);

      isImage ? reqPayload = {
        type: _messageType.MESSAGE_TYPES.IMAGE,
        content: {
          url: null
        },
        externalId: externalId
      } : reqPayload = {
        type: _messageType.MESSAGE_TYPES.FILE,
        content: {
          name: file.name,
          url: null,
          type: (0, _get3.default)(file, ['originalFile', 'type']),
          size: (0, _get3.default)(file, ['originalFile', 'size'])
        },
        externalId: externalId
      };

      listUUIDOrder = isEmpty(listUUIDOrder) ? [externalId] : [].concat(listUUIDOrder, [externalId]);
      dispatch(sendAttachmentRequest(reqPayload));
    });

    form.append('listUUIDOrder', (0, _join3.default)(listUUIDOrder, ','));
    if (!isEmpty(text)) {
      form.append('type', _messageType.MESSAGE_TYPES.TEXT);
      form.append('content', text);

      var reqPayload = { type: _messageType.MESSAGE_TYPES.TEXT, content: { text: text } };
      dispatch(sendAttachmentRequest(reqPayload));
    }
    // Submit form data
    var host = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.HOST, channelKey));
    var request = new XMLHttpRequest();
    request.open('POST', host + '/conversations/messages-from-chatbox');

    request.onreadystatechange = function () {
      if (request.readyState === 4) {
        var res = JSON.parse((0, _get3.default)(request, 'response'));
        var error = (0, _get3.default)(res, 'error');
        if (error) {
          dispatch(sendAttachmentError({ res: res }));
          return;
        }
        dispatch(sendAttachmentSuccess({ res: res }));
      }
    };
    request.send(form);
  };
};

var sendButtonMessage = exports.sendButtonMessage = function sendButtonMessage(buttonData) {
  return function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(dispatch, store) {
      var externalId, reqPayload;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              externalId = new Date().getTime();
              reqPayload = {
                type: _messageType.MESSAGE_TYPES.TEXT,
                content: { text: (0, _get3.default)(buttonData, 'label') },
                externalId: externalId
              };
              _context2.next = 4;
              return dispatch(sendTextMessageRequest(reqPayload));

            case 4:
              (0, _websocket.sendMessage)({
                group: _websocket2.GROUP.MESSAGE,
                type: _websocket2.TYPE.MESSAGE.NEW_MESSAGE_FROM_CHATBOX,
                payload: {
                  type: 'button',
                  content: buttonData,
                  externalId: externalId
                },
                tenantInfo: (0, _tenantInfo.getTenantInfo)()
              });

            case 5:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function (_x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }();
};

var _sendMessage = function _sendMessage(payload) {
  (0, _websocket.sendMessage)({
    group: _websocket2.GROUP.MESSAGE,
    type: _websocket2.TYPE.MESSAGE.UPDATE_MESSAGE_STATUS,
    payload: payload,
    tenantInfo: (0, _tenantInfo.getTenantInfo)()
  });
};

var _updateLastMessageStatus = function _updateLastMessageStatus(_ref3) {
  var lastMessage = _ref3.lastMessage,
      getState = _ref3.getState,
      dispatch = _ref3.dispatch;

  if (!lastMessage) return;
  var lastMessageReceiveStatus = (0, _get3.default)(lastMessage, 'status');
  var senderType = (0, _get3.default)(lastMessage, 'senderType');
  var showChat = (0, _get3.default)(getState(), 'behavior.showChat');
  if (senderType !== 1 && !(0, _includes3.default)([_messageStatus.MESSAGE_STATUS.READ, _messageStatus.MESSAGE_STATUS.FAILED], lastMessageReceiveStatus) && showChat && !document.hidden) {
    _sendMessage((0, _extends3.default)({}, lastMessage, { status: _messageStatus.MESSAGE_STATUS.READ }));
    dispatch(resetNumberOfNewMessages());
  }
};

var getMessagesRequest = exports.getMessagesRequest = (0, _reduxActions.createAction)('GET_MESSAGES_REQUEST');
var getMessagesSuccess = exports.getMessagesSuccess = (0, _reduxActions.createAction)('GET_MESSAGES_SUCCESS');
var getMessagesError = exports.getMessagesError = (0, _reduxActions.createAction)('GET_MESSAGES_ERROR');
var getMessages = exports.getMessages = function getMessages(isLoadingMessageHistory, channelKey) {
  return function () {
    var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(dispatch, getState) {
      var isSimulator, persistentState, conversationId, tenantId, channelId, host, messages, reqPayload, request;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              isSimulator = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.IS_SIMULATOR, channelKey));
              persistentState = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.STORE_STATE, channelKey));
              conversationId = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.CONVERSATION_ID, channelKey));
              tenantId = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.TENANT_ID, channelKey));
              channelId = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.CHANNEL_ID, channelKey));
              host = (0, _localStorage.getLSVariable)((0, _localStorage2.getLocalStorageVariables)(_localStorage2.LOCAL_STORAGE_VARIABLES.HOST, channelKey));
              messages = (0, _get3.default)(persistentState, ['messages', 'messages']);
              reqPayload = {
                conversationId: conversationId,
                tenantId: tenantId,
                channelId: channelId,
                isLoadingMessageHistory: isLoadingMessageHistory
              };

              dispatch(getMessagesRequest(reqPayload));

              request = new XMLHttpRequest();

              request.open('GET', host + '/messages/widget_messages?conversationId=' + conversationId + '&channelId=' + channelId + '&tenantId=' + tenantId + '&pageSize=10' + (isLoadingMessageHistory ? '&skip=' + (0, _size3.default)(messages) : ''));

              request.onreadystatechange = function () {
                if (request.readyState === 4) {
                  var res = JSON.parse((0, _get3.default)(request, 'response'));
                  var error = (0, _get3.default)(res, 'error');
                  if (error) {
                    dispatch(getMessagesError({ res: res }));
                    return;
                  }
                  dispatch(getMessagesSuccess(res, reqPayload));
                  var lastMessage = res[0];
                  !isLoadingMessageHistory && lastMessage && _updateLastMessageStatus({ lastMessage: lastMessage, getState: getState, dispatch: dispatch });
                }
              };
              request.send();

            case 13:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    }));

    return function (_x5, _x6) {
      return _ref4.apply(this, arguments);
    };
  }();
};

var resetNumberOfNewMessages = exports.resetNumberOfNewMessages = (0, _reduxActions.createAction)('RESET_NUMBER_OF_NEW_MESSSAGES');