'use strict';

exports.__esModule = true;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _findIndex2 = require('lodash/findIndex');

var _findIndex3 = _interopRequireDefault(_findIndex2);

var _behaviorReducer;

var _store = require('../../commons/utils/store');

var _actionType = require('../actions/action-type');

var actionTypes = _interopRequireWildcard(_actionType);

var _actions = require('../actions');

var _constants = require('../../components/phone-call/constants');

var _webrtc = require('../../components/phone-call/webrtc');

var _webrtc2 = _interopRequireDefault(_webrtc);

var _updateCall = require('../actions/update-call.action');

var _websocket = require('../../commons/constants/websocket');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isFacebookApp() {
  var ua = navigator.userAgent || navigator.vendor || window.opera;
  return ua.indexOf('FBAN') > -1 || ua.indexOf('FBAV') > -1;
}

var initialState = {
  showChat: false,
  showSupportWidget: false,
  isStartConversationFromAdmin: false,
  hasTriggeredWelcomeMessage: false,
  lastTimeTriggeredWelcomeMessage: undefined,
  lastTimeAcceptChattingWithBot: undefined,
  lastTimeAcceptLeavingOfflineMessage: undefined,
  lastTimeReceivingMessageFromAdmin: undefined,
  alreadySubmitUserInfo: false,
  alreadyStartConversation: false,
  callInfo: {
    selectedSpeakerId: undefined,
    selectedMicrophoneId: undefined,
    isShowCall: false,
    completeCheckingDevice: false,
    isAllowAccessMicrophone: false,
    isDeviceSupport: false,
    devices: [],
    missedDevices: [],
    userRejectsPermission: undefined
  },
  userInfo: {
    name: undefined,
    email: undefined,
    phone: undefined
  },
  soundOn: true,
  numberOfNewMessages: 0,
  triggerStartingPointTracking: []
};

var behaviorReducer = (_behaviorReducer = {}, _behaviorReducer[_actions.toggleTheSupportWidgetRequest] = function (state, _ref) {
  var payload = _ref.payload;

  var status = (0, _get3.default)(state, 'callInfo.callData.status');
  var isCalling = status === _constants.CALL_STATUS.INITIATED || status === _constants.CALL_STATUS.IN_PROGRESS || status === _constants.CALL_STATUS.RECONNECTING;

  // Don't change if calling
  if (isCalling) {
    return state;
  }

  return (0, _extends3.default)({}, state, {
    showSupportWidget: payload,
    showChat: payload === false ? false : state.showChat,
    callInfo: (0, _extends3.default)({}, state.callInfo || {}, {
      isShowCall: payload === false ? false : (0, _get3.default)(state, 'callInfo.isShowCall'),
      callData: undefined
    })
  });
}, _behaviorReducer[_actions.toggleChatRequest] = function (state) {
  return (0, _extends3.default)({}, state, {
    showChat: !state.showChat
  });
}, _behaviorReducer[_actions.startConversationRequest] = function (state, _ref2) {
  var _ref2$payload = _ref2.payload,
      fromButton = _ref2$payload.fromButton,
      isWorkingDay = _ref2$payload.isWorkingDay;

  return (0, _extends3.default)({}, state, {
    alreadyStartConversation: true,
    hasTriggeredWelcomeMessage: fromButton === _websocket.BUTTON_STARTS.START_CONVERSATION && isWorkingDay ? true : state.hasTriggeredWelcomeMessage,
    lastTimeAcceptChattingWithBot: fromButton === _websocket.BUTTON_STARTS.CHAT_WITH_BOT ? new Date() : state.lastTimeAcceptChattingWithBot,
    lastTimeAcceptLeavingOfflineMessage: fromButton === _websocket.BUTTON_STARTS.LEAVE_MESSAGE ? new Date() : state.lastTimeAcceptLeavingOfflineMessage
  });
}, _behaviorReducer[_actions.updateHasTriggeredWelcomeMessage] = function (state) {
  return (0, _extends3.default)({}, state, {
    hasTriggeredWelcomeMessage: true,
    lastTimeTriggeredWelcomeMessage: new Date()
  });
}, _behaviorReducer[_actions.saveUserInfoRequest] = function (state, _ref3) {
  var payload = _ref3.payload;

  return (0, _extends3.default)({}, state, {
    userInfo: (0, _extends3.default)({}, state.userInfo, {
      name: (0, _get3.default)(payload, 'name'),
      email: (0, _get3.default)(payload, 'email'),
      phone: (0, _get3.default)(payload, 'phone')
    }),
    alreadySubmitUserInfo: true
  });
}, _behaviorReducer[_actions.trackingTheTriggerEvent] = function (state, _ref4) {
  var _ref4$payload = _ref4.payload,
      startingPointType = _ref4$payload.startingPointType,
      startingPointId = _ref4$payload.startingPointId;

  console.log('trackingTheTriggerEvent: ');
  var existOneIndex = (0, _findIndex3.default)(state.triggerStartingPointTracking, function (event) {
    return event.startingPointId === startingPointId;
  });
  var triggerStartingPointTracking = state.triggerStartingPointTracking || [];
  if (existOneIndex >= 0) {
    triggerStartingPointTracking[existOneIndex] = (0, _extends3.default)({}, triggerStartingPointTracking[existOneIndex], {
      lastTime: new Date()
    });
  } else {
    triggerStartingPointTracking.push({
      startingPointType: startingPointType,
      startingPointId: startingPointId,
      lastTime: new Date()
    });
  }

  var now = new Date();
  var oneDayInMilleseconds = 1000 * 60 * 60 * 24;
  return (0, _extends3.default)({}, state, {
    triggerStartingPointTracking: triggerStartingPointTracking.filter(function (event) {
      return Math.abs(now - event.lastTime) < oneDayInMilleseconds;
    })
  });
}, _behaviorReducer[actionTypes.TOGGLE_CALL] = function (state) {
  // console.log("toggleCall");
  var isShowing = (0, _get3.default)(state, 'callInfo.isShowCall');
  var status = (0, _get3.default)(state, 'callInfo.callData.status');
  var isCalling = status === _constants.CALL_STATUS.INITIATED || status === _constants.CALL_STATUS.IN_PROGRESS || status === _constants.CALL_STATUS.RECONNECTING;

  if (isCalling) {
    return state;
  }

  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      isShowCall: !isShowing,
      callData: undefined
    })
  });
}, _behaviorReducer[_actions.updateMediaPermission] = function (state, _ref5) {
  var payload = _ref5.payload;

  var isAllow = payload;
  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      isAllowAccessMicrophone: isAllow,
      userRejectsPermission: !isAllow
    })
  });
}, _behaviorReducer[_actions.checkMediaDeviceSuccess] = function (state, _ref6) {
  var payload = _ref6.payload;

  // console.log("payload", payload);
  var totalAudioInput = payload.totalAudioInput,
      totalAudioOutput = payload.totalAudioOutput,
      devices = payload.devices,
      isAllowAcessMedia = payload.isAllowAcessMedia;


  var getMissedDevices = function getMissedDevices() {
    var results = [];
    if (totalAudioInput === 0) {
      results.push('input');
    }

    if (totalAudioOutput === 0) {
      results.push('output');
    }
  };
  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      devices: devices,
      isAllowAccessMicrophone: isAllowAcessMedia,
      isDeviceSupport: !/iphone|ipod|ipad|ucbrowser/.test(window.navigator.userAgent.toLowerCase()) && !isFacebookApp() && (totalAudioInput > 0 || totalAudioOutput > 0),
      missedDevices: getMissedDevices()
    })
  });
}, _behaviorReducer[_actions.startCallRequest] = function (state, _ref7) {
  var payload = _ref7.payload;

  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      callData: {
        status: _constants.CALL_STATUS.CONNECTING,
        subscriberPeerConnectionInfo: (0, _get3.default)(payload, 'subscriberPeerConnectionInfo')
      }
    })
  });
}, _behaviorReducer[_actions.startCallSuccess] = function (state, _ref8) {
  var payload = _ref8.payload;

  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      callData: {
        status: _constants.CALL_STATUS.QUEUED,
        subscriberPeerConnectionInfo: (0, _get3.default)(payload, 'subscriberPeerConnectionInfo')
      }
    })
  });
}, _behaviorReducer[_actions.startCallError] = function (state, _ref9) {
  var payload = _ref9.payload;

  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      callData: {
        error: (0, _get3.default)(payload, 'error'),
        status: _constants.CALL_STATUS.FAILED
      }
    })
  });
}, _behaviorReducer[_actions.receiveICEServerResponse] = function (state, _ref10) {
  var payload = _ref10.payload;

  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      callData: (0, _extends3.default)({}, state.callInfo.callData, {
        iceServers: payload
      })
    })
  });
}, _behaviorReducer[_actions.receiveUpdateCallStatus] = function (state, _ref11) {
  var payload = _ref11.payload;

  // console.log("receiveUpdateCallStatus: ", payload);
  var status = (0, _get3.default)(payload, ['status']);
  if (status === _constants.CALL_STATUS.COMPLETED || status === _constants.CALL_STATUS.ADMIN_CANCELED || status === _constants.CALL_STATUS.BUSY || status === _constants.CALL_STATUS.NO_ANSWER) {
    // console.log("STOP CALL");
    _webrtc2.default.stopCall();
  }
  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      callData: payload
    })
  });
}, _behaviorReducer[_updateCall.updateCallInfoRequest] = function (state, _ref12) {
  var payload = _ref12.payload;

  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      callData: (0, _extends3.default)({}, state.callInfo.callData, payload)
    })
  });
}, _behaviorReducer[_actions.leaveInformationSuccess] = function (state, _ref13) {
  var payload = _ref13.payload;

  return (0, _extends3.default)({}, state, {
    userInfo: (0, _extends3.default)({}, state.userInfo, {
      name: (0, _get3.default)(payload, 'name'),
      email: (0, _get3.default)(payload, 'email'),
      phone: (0, _get3.default)(payload, 'phone')
    }),
    alreadySubmitUserInfo: true
  });
}, _behaviorReducer[_actions.selectDevices] = function (state, _ref14) {
  var _ref14$payload = _ref14.payload,
      speakerId = _ref14$payload.speakerId,
      microphoneId = _ref14$payload.microphoneId;

  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      selectedSpeakerId: speakerId,
      selectedMicrophoneId: microphoneId
    })
  });
}, _behaviorReducer[_actions.stopCallRequest] = function (state, _ref15) {
  var payload = _ref15.payload;

  return (0, _extends3.default)({}, state, {
    callInfo: (0, _extends3.default)({}, state.callInfo, {
      callData: (0, _extends3.default)({}, state.callInfo.callData, {
        status: payload.status
      })
    })
  });
}, _behaviorReducer[_actions.receiveMessageFromAdmin] = function (state, _ref16) {
  var payload = _ref16.payload;

  return (0, _extends3.default)({}, state, {
    isStartConversationFromAdmin: state.alreadyStartConversation === false,
    lastTimeReceivingMessageFromAdmin: new Date()
  });
}, _behaviorReducer[actionTypes.SOUND_SETTING] = function (state) {
  return (0, _extends3.default)({}, state, {
    soundOn: !state.soundOn
  });
}, _behaviorReducer);

exports.default = function () {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments[1];
  return (0, _store.createReducer)(behaviorReducer, state, action);
};

module.exports = exports['default'];