'use strict';

exports.__esModule = true;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _findLastIndex2 = require('lodash/findLastIndex');

var _findLastIndex3 = _interopRequireDefault(_findLastIndex2);

var _size2 = require('lodash/size');

var _size3 = _interopRequireDefault(_size2);

var _unionBy2 = require('lodash/unionBy');

var _unionBy3 = _interopRequireDefault(_unionBy2);

var _find2 = require('lodash/find');

var _find3 = _interopRequireDefault(_find2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _pick2 = require('lodash/pick');

var _pick3 = _interopRequireDefault(_pick2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

var _messageReducer;

var _store = require('../../commons/utils/store');

var _correctMessage = require('../../commons/utils/correct-message');

var _messageStatus = require('../../commons/constants/message-status');

var _senderTypes = require('../../commons/constants/sender-types');

var _actions = require('../actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MAX_NUMBER_OF_MESSAGES = 20;
var defaultIdForSubscriber = 'subscriber';
var defaultIdForBot = 'bot';
var getSenderId = function getSenderId(message) {
  if (!message) return;
  if (message.senderType === _senderTypes.SENDER_TYPES.BOT) {
    return defaultIdForBot;
  }

  if (message.senderType === _senderTypes.SENDER_TYPES.SUBSCRIBER) {
    return defaultIdForSubscriber;
  }

  return message.adminId || defaultIdForBot;
};
function getMessageFromPayload(payload) {
  var type = (0, _get3.default)(payload, 'type');
  if (type === 'text') {
    return (0, _extends3.default)({}, (0, _pick3.default)(payload, ['id', 'type', 'status', 'externalId']), {
      content: (0, _get3.default)(payload, 'content.text') || (0, _get3.default)(payload, 'content')
    });
  }
  return (0, _pick3.default)(payload, ['id', 'type', 'status', 'content', 'externalId']);
}

function shortifyMessages(messages) {
  // return messages;
  if (!messages) return [];
  if ((0, _size3.default)(messages) > MAX_NUMBER_OF_MESSAGES) {
    // console.log('messages before: ', size(messages));
    var results = messages.slice((0, _size3.default)(messages) - MAX_NUMBER_OF_MESSAGES);
    // console.log('messages after: ', size(results));
    return results;
  }
  return messages;
}

function mergeOrAppendNewMessages(oldMessages, newMessages) {
  var result = (0, _unionBy3.default)(oldMessages, newMessages, function (m) {
    return m.message.id;
  }).sort(function (a, b) {
    return a.message.id - b.message.id;
  });
  // console.log('mergeOrAppendNewMessages: ');
  // console.log('newMessages: ', JSON.stringify(newMessages));
  // console.log('oldMessages: ', JSON.stringify(oldMessages));
  // Merge new fields to the existing one

  var _loop = function _loop(i, item) {
    if (item.message.id) {
      var foundItem = (0, _find3.default)(newMessages, function (m) {
        return m.message.id === item.message.id;
      });
      if (foundItem) {
        // console.log('found item: ', item.message.id);
        result[i] = (0, _extends3.default)({}, item, {
          senderType: (0, _get3.default)(foundItem, 'senderType') || (0, _get3.default)(item, 'senderType'), // For hot fix
          message: (0, _extends3.default)({}, item.message, foundItem.message)
        });
      } else {
        // console.log('not found: ', item.message.id);
      }
    }
  };

  for (var _iterator = result.entries(), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref2;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref2 = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref2 = _i.value;
    }

    var _ref = _ref2;
    var i = _ref[0];
    var item = _ref[1];

    _loop(i, item);
  }

  return (0, _correctMessage.handleCorrectMessages)(result);
}

function getNumberOfNewMessages(messages) {
  var numberOfNewMessages = 0;
  var lastMessageRead = (0, _findLastIndex3.default)(messages, ['status', _messageStatus.MESSAGE_STATUS.READ]);
  (messages || []).forEach(function (m, index) {
    if (index > lastMessageRead && m.senderId !== 'subscriber' && (0, _get3.default)(m, ['message', 'type']) !== 'delay') {
      numberOfNewMessages = numberOfNewMessages + 1;
    }
  });
  return numberOfNewMessages;
}

var initialState = {
  rawMessages: [],
  messages: [],
  participants: [{
    id: defaultIdForBot,
    name: 'bot',
    avatar: '',
    side: 'left',
    isShowAvatar: true
  }, {
    id: defaultIdForSubscriber,
    name: 'user01',
    avatar: '',
    side: 'right',
    isShowAvatar: false
  }],
  isLoadingMessage: false,
  isLoadingMessageHistory: false,
  numberOfNewMessages: 0
};

var messageReducer = (_messageReducer = {}, _messageReducer[_actions.onScrollToBottom] = function (state) {
  // console.log('onScrollToBottom');
  if (state.isLoadingMessageHistory) {
    return state;
  }
  return (0, _extends3.default)({}, state, { messages: shortifyMessages(state.messages) });
}, _messageReducer[_actions.toggleChatRequest] = function (state, _ref3) {
  var payload = _ref3.payload;

  return (0, _extends3.default)({}, state, {
    numberOfNewMessages: payload === true ? 0 : getNumberOfNewMessages(state.messages)
  });
}, _messageReducer[_actions.sendTextMessageRequest] = function (state, _ref4) {
  var payload = _ref4.payload;

  var newMessages = [].concat(state.messages, [{
    senderId: defaultIdForSubscriber,
    senderType: _senderTypes.SENDER_TYPES.SUBSCRIBER,
    message: getMessageFromPayload(payload)
  }]);

  return (0, _extends3.default)({}, state, {
    messages: newMessages,
    numberOfNewMessages: getNumberOfNewMessages(newMessages)
  });
}, _messageReducer[_actions.receiveCreatedMessageNotification] = function (state, _ref5) {
  var payload = _ref5.payload;
  var id = payload.id,
      status = payload.status,
      message = payload.message;

  return (0, _extends3.default)({}, state, {
    messages: state.messages.map(function (item) {
      if (item.message.id) return item;
      if ((0, _get3.default)(item, 'message.externalId') === (0, _get3.default)(message, 'externalId')) {
        return (0, _extends3.default)({}, item, {
          message: getMessageFromPayload((0, _extends3.default)({ id: id, status: status }, message))
        });
      }
      return item;
    })
  });
}, _messageReducer[_actions.receiveMessageFromAdmin] = function (state, _ref6) {
  var payload = _ref6.payload;

  var messages = mergeOrAppendNewMessages(state.messages, [{
    senderId: defaultIdForBot,
    senderType: (0, _get3.default)(payload, 'sender.isBot') ? _senderTypes.SENDER_TYPES.BOT : _senderTypes.SENDER_TYPES.ADMIN,
    message: getMessageFromPayload(payload)
  }]);

  return (0, _extends3.default)({}, state, {
    messages: messages,
    // messages: shortifyMessages(messages),
    numberOfNewMessages: getNumberOfNewMessages(messages)
  });
}, _messageReducer[_actions.sendAttachmentRequest] = function (state, _ref7) {
  var payload = _ref7.payload;

  return (0, _extends3.default)({}, state, {
    messages: [].concat(state.messages, [{
      senderId: defaultIdForSubscriber,
      senderType: _senderTypes.SENDER_TYPES.SUBSCRIBER,
      message: getMessageFromPayload(payload)
    }])
  });
}, _messageReducer[_actions.sendAttachmentSuccess] = function (state, action) {
  // Old messages
  var listMessages = [].concat(state.messages);
  // New messages
  var newMessages = (0, _get3.default)(action, ['payload', 'res']);
  // Append new messages
  (newMessages || []).map(function (newMessage) {
    var externalId = (0, _get3.default)(newMessage, 'externalId');
    listMessages = listMessages.map(function (mes) {
      var originalExternalId = (0, _get3.default)(mes, ['message', 'externalId']);
      if (Number(originalExternalId) === Number(externalId)) {
        return {
          senderId: defaultIdForSubscriber,
          senderType: _senderTypes.SENDER_TYPES.SUBSCRIBER,
          message: getMessageFromPayload((0, _get3.default)(newMessage, 'message'))
        };
      }
      return mes;
    });
  });
  return (0, _extends3.default)({}, state, {
    messages: listMessages
  });
}, _messageReducer[_actions.sendAttachmentError] = function (state, action) {
  return (0, _extends3.default)({}, state);
}, _messageReducer[_actions.getMessagesRequest] = function (state, _ref8) {
  var payload = _ref8.payload;

  return (0, _extends3.default)({}, state, {
    isLoadingMessage: true,
    isLoadingMessageHistory: payload.isLoadingMessageHistory
  });
}, _messageReducer[_actions.getMessagesSuccess] = function (state, _ref9) {
  var payload = _ref9.payload;

  var messages = payload || [];
  if ((0, _isEmpty3.default)(messages)) return (0, _extends3.default)({}, state, {
    isLoadingMessage: true,
    isLoadingMessageHistory: false,
    numberOfNewMessages: getNumberOfNewMessages(state.message)
  });

  var newMessages = mergeOrAppendNewMessages(state.messages, messages.map(function (m) {
    return {
      senderId: getSenderId(m),
      senderType: m.senderType,
      message: getMessageFromPayload((0, _extends3.default)({ id: m.id, status: m.status }, m.message))
    };
  }));
  return (0, _extends3.default)({}, state, {
    isLoadingMessage: true,
    isLoadingMessageHistory: false,
    messages: newMessages,
    numberOfNewMessages: getNumberOfNewMessages(newMessages)
  });
}, _messageReducer[_actions.resetNumberOfNewMessages] = function (state) {
  var lastMessage = state.messages.pop();
  return (0, _extends3.default)({}, state, {
    messages: [].concat(state.messages, [(0, _extends3.default)({}, lastMessage, { status: _messageStatus.MESSAGE_STATUS.READ })]),
    numberOfNewMessages: 0
  });
}, _messageReducer);

exports.default = function () {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments[1];
  return (0, _store.createReducer)(messageReducer, state, action);
};

module.exports = exports['default'];